// Datei Browser01.java
/* ------------------------------------------------------------------------
Ein simpler Browser. Der Benutzer kann eine Internet-Adresse eingeben (z.B.
http://public.tfh-berlin.de oder file://localhost/c:/autoexec.bat) und die
entsprechende Seite wird angezeigt. Nicht ganz so leistungsfaehig wie der
InternetExplorer, dafuer deutlich schlanker und durchschaubarer :-).
Nach einem Beispiel aus: J.L. Weber: "Using Java2 Platform", Verlag Que 1999
------------------------------------------------------------------------ */
import javax.swing      .JFrame;
import javax.swing      .JEditorPane;
import javax.swing      .JTextField;
import javax.swing      .JScrollPane;
import javax.swing.event.HyperlinkListener;
import javax.swing.event.HyperlinkEvent;
import java .awt  .event.ActionListener;
import java .awt  .event.ActionEvent;
import java .io         .IOException;
import java .net        .URL;
import java .net        .MalformedURLException;
// ------------------------------------------------------------------------
public class Browser01 extends JFrame {
   final protected JEditorPane eineNetzSeite;
   final protected JTextField  eingabeZeile;
   final protected JTextField  ausgabeZeile;
   // ----------------------------------------------------------------------
   public Browser01(String titel){
      super(titel);
      setDefaultCloseOperation(EXIT_ON_CLOSE);
      // ------------------------------------------------------------------
      // Das JEditorPane-Objekt erzeugen etc., in dem dann jeweils eine
      // Seite aus dem Netzt angezeigt wirde:
      eineNetzSeite = new JEditorPane();
      eineNetzSeite.setEditable(false);
      eineNetzSeite.addHyperlinkListener(new HyperlinkListener(){
         // Wenn der Benutzer die Maus in den Bereich eines Links
         // hineinbewegt (ENTERED) oder aus dem Bereich hinausbewegt
         // (EXITED) oder auf einen Link klickt (ACTIVATED), wir die
         // folgende Methode ausgefuehrt:
         public void hyperlinkUpdate(HyperlinkEvent linkEreignis){
            HyperlinkEvent.
            EventType ereignisTyp = linkEreignis.getEventType();
            URL       ereignisURL = linkEreignis.getURL();

            // Den Ereignis-Typ (ENTERED, EXITED oder ACTIVATED) und das
            // URL-Objekt des Link-Ereignisses ausgeben
            printf("Link-Ereignis %-10s %s%n", ereignisTyp, ereignisURL);

            // Wenn die Maus in den Bereich eines Links HINEINbewegt
            // wurde, wird der Link in der ausgabeZeile angezeigt:
            if (ereignisTyp == HyperlinkEvent.EventType.ENTERED) {
               ausgabeZeile.setText(""+ereignisURL);
            }

            // Wenn die Maus aus dem Bereich eines Links HINAUSbewegt
            // wurde, wird die ausgabeZeile geloescht:
            if (ereignisTyp == HyperlinkEvent.EventType.EXITED) {
               ausgabeZeile.setText("");
            }

            // Wenn der Benutzer auf einen Link geklickt hat, dann die
            // entsprechende Seite anzeigen:
            if (ereignisTyp == HyperlinkEvent.EventType.ACTIVATED) {
               eingabeZeile.setText(""+ereignisURL);
               setURL(ereignisURL);
            }
         } // hyperlinkUpdate
      }); // eineNetzSeite.addHyperlinkListener

      // Ein Skroll-Fenster erzeugen und eineNetzSeite hineintun:
      JScrollPane jsp = new JScrollPane(eineNetzSeite);
      // Das Skroll-Fenster in das aktuelle Browser01-Objekt einfuegen:
      this.getContentPane().add(jsp,"Center");
      // ------------------------------------------------------------------
      // Das JTextField-Objekt erzeugen etc., in das der Benutzer eine
      // Netz-Adresse (z.B. "http://www.tfh-berlin.de") eingeben kann:
      eingabeZeile = new JTextField(30); // 30 Spalten breit

      eingabeZeile.addActionListener(new ActionListener(){
         // Wenn der Benutzer in der Eingabezeile auf die Return-Taste
         // drueckt wird die folgende Methode ausgefuehrt:
         public void actionPerformed(ActionEvent ae){
           try {
             URL neuesUrlObjekt = new URL(ae.getActionCommand());
             printf("Neue Netz-Adresse: %s%n", neuesUrlObjekt);
             setURL(neuesUrlObjekt);
           } catch (MalformedURLException mue){
             printf("Falsche URL?: %s%n", mue);
           }
         } // actionPerformed
      });
      this.getContentPane().add(eingabeZeile,"North");
      // ------------------------------------------------------------------
      // Das JTextField-Objekt erzeugen etc., in dem Links angezeigt werden:
      ausgabeZeile = new JTextField(30); // 30 Spalten breit
      ausgabeZeile.setEditable(false);
      this.getContentPane().add(ausgabeZeile, "South");
      // ------------------------------------------------------------------
      try {
         String startString = "http://www.google.de";
         URL    startURL    = new URL(startString);
         eingabeZeile.setText(startString);
         setURL(startURL);
      } catch (MalformedURLException murle) {
         printf("Ausnahme: %s%n", murle);
      }
      // ------------------------------------------------------------------
      this.setBounds(200, 200, 800, 500);
      this.setVisible(true);
      // ------------------------------------------------------------------
   } // Konstruktor Browser01
   // ----------------------------------------------------------------------
   public void setURL(URL neuesUrlObjekt){
      try {
         // Das JEditorPane-Objekt eineNetzSeite wird mit der neuen URL
         // verbunden. Das Objekt versucht, festzustellen, was fuer einen
         // Inhalts-Typ die Seite hat und zeigt die Seite an:
         eineNetzSeite.setPage(neuesUrlObjekt);
      } catch (IOException ioe){
         printf("Ausnahme : %s%n", ioe);
      } // try/catch
   } // setURL
   // ----------------------------------------------------------------------
   static public void main(String[] _) {
      printf("Browser01: Jetzt geht es los!%n");
      new Browser01(" Browser01");
   } // main
   // ----------------------------------------------------------------------
   // Eine Methode mit einem kurzen Namen:
   static void printf(String f, Object... v) {System.out.printf(f, v);}
   // ----------------------------------------------------------------------
} // class Browser01
// ------------------------------------------------------------------------