// Datei DialogGeneratorFenster.java
/* ------------------------------------------------------------------------
Mit diesem Programm kann man mehrere Primzahlpruefungen nebenlaeufig
zueinander durchfuehren lassen. Dadurch soll der Sinn von Faeden (engl.
threads) in GRABO-Programmen (engl. GUI programs) veranschaulicht werden.

Wenn man das Programm startet, erscheint auf dem Bildschirm ein kleines
DialogGenerator-Fenster mit einem "Neuer Dialog"-Knopf darin. Jedesmal
wenn man auf diesen Knopf klickt, wird ein neues kleines Dialog-Fenster
erzeugt und angezeigt (aber wenn man mehr als 30 Dialog-Fenster erzeugt,
leidet dadurch die Uebersichtlichkeit auf dem Bildschirm :-).

Jedes Dialog-Fenster enthaelt zwei Textzeilen. In die obere sollte man
eine Ganzzahl eintragen (z.B. 12 oder 17 oder 1.234.567.811) und dann
auf Return druecken. Daraufhin wird (von einem neuen Faden) geprueft,
ob die eingegebene Zahl prim ist oder nicht. Das Ergebnis der Pruefung
wird in der unteren Textzeile angezeigt, z.B. so: "istPrim(12): false"
oder "istPrim(17): true" etc.

Fuer 14 bestimmte, relativ grosse Zahlen gibt es "Kurzwahlnummern":
Wenn man eine Zahl zwischen 1 und 7 oder zwischen -1 und -7 eingibt, wird
sie durch eine der 14 grossen Zahlen ersetzt, bevor die Pruefung beginnt.
Das Pruefen einer solchen grossen Zahl dauert (mit dem hier verwendeten
relativ langsamen Algorithmus) auf heute ueblichen Rechnern viele Sekunden
bis ein paar Minuten.

Waehrend in einem Dialog-Fenster eine Zahl geprueft wird, kann man weitere
Dialog-Fenster erzeugen und/oder in schon vorhandenen Dialog-Fenstern
weitere Zahlen eingeben und pruefen lassen. Das ist nur moeglich, weil die
zeitaufwendigen Primzahlpruefungen von zusaetzlichen Faeden durchgefuehrt
werden (und nicht vom Hauptfaden oder vom Ereignisfaden des Programms).

Alle Primzahlpruefungen werden durch Ausgaben zur Standardausgabe proto-
kolliert. Das Protokoll beginnt mit einer kurzen Bedienungsanleitung und
den 14 grossen Zahlen, fuer die es "Kurzwahlnummern" gibt.

Hinweis: Als "Trick" kann man mit *einem* Dialog-Fenster *mehrere* Faeden
starten, die nebenlaeufig zueinander ausgefuehrt werden, etwa so: Man gibt
im selben Dialog-Fenster (zuegig) nacheinander die Kurzwahlnummern 3, 2
und 1 ein (und schliesst jede Eingabe wie immer mit Return ab). An den
Protokollausgaben zum Bildschirm kann man dann (hoffentlich) erkennen,
dass die Pruefungen dieser Zahlen nebenlaeufig zueinander ausgefuehrt
wurden (obwohl man nur ein Dialog-Fenster benutzt hat).
------------------------------------------------------------------------ */
import javax.swing       .Box;
import javax.swing       .BoxLayout;
import javax.swing       .JButton;
import javax.swing       .JDialog;
import javax.swing       .JFrame;
import javax.swing       .JTextField;
import javax.swing.border.LineBorder;

import java .awt  .event .ActionListener;
import java .awt  .event .ActionEvent;
import java .awt         .BorderLayout;
import java .awt         .Color;
import java .awt         .Font;
import java .awt         .Container;

import java .io          .PrintWriter;
import java .io          .StringWriter;

import java .text        .DecimalFormat;
import java .text        .ParsePosition;

import java .util        .GregorianCalendar;
import java .util        .ArrayList;
import java .util        .Arrays;
import java .util        .List;
import java .util        .Map;
import java .util        .HashMap;
import java .util        .Iterator;

class DialogGeneratorFenster extends JFrame {
   // Von dieser Klasse wird nur ein Objekt erzeugt. Es wird auf dem
   // Bildschirm als kleines Fenster mit dem Titel "Dialog-Generator" und
   // einem Knopf "Neuer Dialog"- Knopf in der Fenstermitte angezeigt.

   // Diese Klasse
   // DialogGeneratorFenster      enthaelt 5 innere Klassen namens
   // DialogGeneratorLauscher,
   // DialogFenster,
   // DialogLauscher,
   // ArbeitsFaden und
   // FortschrittsDaemon.
   // ---------------------------------------------------------------------
   // Konstanten und Variablen, die die Grenzen (Lage und Groesse) aller
   // Fenster und des "Neuer Dialog"-Knopfes beschreiben.

   // Ein kleiner Abstand (in x- und y-Richtung) zwischen Fenstern:
   final int d     =   5;

   // Grenzen des (einen) DialogGenerator-Fensters dgf:
   final int dgfSX = 250;               // Seite x
   final int dgfSY = 150;               // Seite y
   final int dgfX  =   d;               // x-Koord. der linken oberen Ecke
   final int dgfY  =   d;               // y-Koord. der linken oberen Ecke

   // Grenzen des "Neuer Dialog"-Knopfes dgk im dgf:
   // (dieser Knopf wird ohne Layout-Manager "von Hand" positioniert):
   final int dgkSX =  100;              // Seite x
   final int dgkSY =   30;              // Seite y
   final int dgkX  = (dgfSX - dgkSX)/2; // x-Koord. der linken oberen Ecke
   final int dgkY  = (dgfSY - dgkSY)/2; // y-Koord. der linken oberen Ecke

   // Grenzen der (moeglicherweise zahlreichen) Dialog-Fenster dif:
   // (die Koordinaten difX und difY werden spaeter
   // noch mit Indizes   iX bzw.  iY multipliziert):
   final int difSX = 350;               // Seite x
   final int difSY =  80;               // Seite y
         int difX  =  dgfX + dgfSX + d; // x-Koord. der linken oberen Ecke
         int difY  =  dgfY;             // y-Koord. der linken oberen Ecke

   // Indizes der (moeglicherweise zahlreichen) Dialog-Fenster dif:
   // Die Dialog-Fenster werden auf dem Bildschirm in einem Gittermuster mit
   // maxIX Spalten und maxIY Zeilen positioniert. Innerhalb dieses Gitters
   // hat jedes Fenster einen
   // Index iX zwischen 0 und maxIX-1 und einen
   // Index iY zwischen 0 und maxIY-1:
   final int maxIX =   3;               //  3 Spalten (iX laeuft von 0 bis 2)
   final int maxIY =  10;               // 10 Zeilen  (iY laeuft von 0 bis 9)
         int iX    =   0;               // Indizes des Dialog-Fensters
         int iY    =   0;               // welches als naechstes erzeugt wird.
   // ---------------------------------------------------------------------
   static public boolean istPrim(long n) {
      // Liefert true, wenn n eine Primzahl ist.
      //
      // Diese Methode steht hier fuer irgendeinen Verarbeitungsschritt,
      // der manchmal ziemlich lange dauert. Desshalb wurden (fast) keine
      // "Beschleunigungen oder Optimierungen" eingebaut.

      n = Math.abs(n);
      if (n<=1) return false;
      for (long t=2; t<=n/2; t++) {
         if (n%t==0) return false;
      }
      return true;
   } // istPrim
   // ---------------------------------------------------------------------
   public DialogGeneratorFenster() {
      // Dieser Konstruktor wird nur einmal aufgerufen:

      super("DialogGenerator");

      // Den "Neuer Dialog"-Knopf erzeugen und mit einer Umrandung und
      // einem ActionListener-Objekt ausruesten:
      JButton    dialogGeneratorKnopf = new JButton("Neuer Dialog");

      dialogGeneratorKnopf.setBorder(new LineBorder(Color.BLACK, 3));
      dialogGeneratorKnopf.addActionListener(new DialogGeneratorLauscher());
      dialogGeneratorKnopf.setBounds(dgkX, dgkY, dgkSX, dgkSY);

      // Den "Neuer Dialog"-Knopf in das DialogGenerator-Fenster
      // einfuegen, die Grenzen des Fensters festlegen und das Fenster
      // sichtbar machen:
      this.getContentPane().setLayout(null);
      this.getContentPane().add(dialogGeneratorKnopf);
      this.setBounds(dgfX, dgfY, dgfSX, dgfSY);
      this.setDefaultCloseOperation(EXIT_ON_CLOSE);
      this.setVisible(true);

      // Den Daemon, der die Fortschrittanzeigen in den Dialog-Fenstern
      // weiterbewegt, erzeugen und starten:
      new FortschrittsDaemon(alleDialogFenster);
   } // Konstruktor DialogGeneratorFenster
   // ---------------------------------------------------------------------
   static public void main(String[] _) {
      // Erzeugt ein DialogGenerator-Fenster. Damit kann der Benutzer
      // beliebig viele Faeden starten und jeden mit der Pruefung einer
      // Ganzzahl (Ist sie prim oder nicht?) beauftragen.

      printf(
         "------------------------------------------------------%n" +
         "DialogGeneratorFenster: Jetzt geht es los!            %n" +
         "                                                      %n" +
         "Bedienungsvorschlag:                                  %n" +
         "Klicken Sie dreimal auf den Knopf 'Neuer Dialog'.     %n" +
         "Geben Sie in jedes der drei sich oeffnenden Dialog-   %n" +
         "Fenster eine der drei Zahlen 2, 1 und -1 ein (jeweils %n" +
         "in die obere Zeile) und schliessen Sie jede Eingabe   %n" +
         "mit Return ab. Warten Sie, bis alle drei Fenster rot  %n" +
         "bzw. gruen geworden sind. Das kann viele  Sekunden    %n" +
         "oder sogar einige Minuten dauern.                     %n" +
         "------------------------------------------------------%n" +
         "'Kurzwahlnummern' fuer bestimmte Zahlen:              %n" +
         "                                                      %n" +
         "  Die                 Steht fuer      Ist eine        %n" +
         " Kurzwahl:          die grosse Zahl:  Primzahl:       %n" +
         "    1                   150.000.449     ja            %n" +
         "    2                   250.000.139     ja            %n" +
         "    3                 1.000.000.021     ja            %n" +
         "    4                 1.234.567.811     ja            %n" +
         "    5                 1.234.567.891     ja            %n" +
         "    6                 2.000.000.087     ja            %n" +
         "    7                 2.000.000.089     ja            %n" +
         "   -1        22.500.134.700.201.601     nein          %n" +
         "   -2        62.500.069.500.019.321     nein          %n" +
         "   -3     1.000.000.042.000.000.441     nein          %n" +
         "   -4     1.524.157.679.957.331.721     nein          %n" +
         "   -5     1.524.157.877.488.187.881     nein          %n" +
         "   -6     4.000.000.348.000.007.569     nein          %n" +
         "   -7     4.000.000.356.000.007.921     nein          %n" +
         "                                                      %n" +
         "Die POSITIVEN Kurzwahlnummern stehen fuer Primzahlen, %n" +
         "die NEGATIVEN fuer die Quadrate diser Primzahlen.     %n" +
         "Beispiel: Die Kurzwahl -3 steht fuer das Qadrat der   %n" +
         "Primzahl mit der Kurzwahl 3.                          %n" +
         "----------------------------------------------------- %n"
      );

      new DialogGeneratorFenster();
   } // main
   // =====================================================================
   // Die innere Klasse FortschrittsDaemon (und ihr "Zubehoer")
   // =====================================================================
   // Als (sehr simple) Forstschrittsanzeige in den Dialog-Fenstern werden
   // zyklisch die folgenden vier Unicode-Zeichen (als erstes Zeichen in
   // einem JTextField-Objekt) angezeigt:
   // \u2502 BOX DRAWINGS LIGHT VERTICAL
   // \u2571 BOX DRAWINGS LIGHT DIAGONAL UPPER RIGHT TO LOWER LEFT
   // \u2500 BOX DRAWINGS LIGHT HORIZONTAL
   // \u2572 BOX DRAWINGS LIGHT DIAGONAL UPPER LEFT TO LOWER RIGHT

   // Weil die Methode setText eines JTextField-Objekts einen String
   // als Parameter erwartet, sind die "Zeichen" hier Strings (und
   // nicht char-Werte oder Character-Objekte):
   final String[]     fzr = {"\u2502", "\u2571", "\u2500", "\u2572"};
   final List<String> fortschrittsZeichen =
      new ArrayList<String>(Arrays.asList(fzr));

   class FortschrittsDaemon extends Thread {
      // Von dieser Klasse wird nur ein Objekt erzeugt und (als Faden)
      // gestartet. Dieser Faden ist ein Daemon, der die Fortschritts-
      // anzeigen in allen DialogFenster-Objekten "weiterbewegt".
      // Der Daemon arbeitet "getaktet": Am Anfang eines Taktes wacht er
      // auf, macht seine Arbeit und schlaeft dann bis zum Anfang des
      // naechstens Taktes.

      final int                DAUER_EINES_TAKTS = 1000; // Millisekunden
      ArrayList<DialogFenster> alleDialogFenster;
      long                     taktAnfang;

      public FortschrittsDaemon(ArrayList<DialogFenster> alleDialogFenster){
         this.alleDialogFenster = alleDialogFenster;
         this.taktAnfang        = System.currentTimeMillis();
         this.setDaemon(true);
         this.start();
      } // Konstruktor FortschrittsDaemon
      // ------------------------------------------------------------------
      public void run() {
         while (true) {

            Iterator<DialogFenster> it = alleDialogFenster.iterator();
            while (it.hasNext()) {
               DialogFenster df = it.next();

               // Falls der Benutzer df geschlossen hat, wird df aus
               // der Sammlung alleDialogFenster entfernt:
               if (! df.isShowing()) {
                  it.remove();
                  continue;
               }

               // Die "Ausgabezeile" aus von df und ihren Inhalt str holen:
               Container  cnt = df.getContentPane();
               Box        box = (Box)        cnt.getComponent(0);
               JTextField aus = (JTextField) box.getComponent(1);
               String     str = aus.getText();

               // Falls der Inhalt ein Fortschrittszeichen ist, wird es
               // durch das naechste Fortschrittszeichen ersetzt:
               int        ind = fortschrittsZeichen.indexOf(str);
               if (ind == -1) continue;
               ind = (ind+1) % fortschrittsZeichen.size();
               aus.setText(fortschrittsZeichen.get(ind));
            }

            // Wartezeit bis zum Anfang des naechsten Taktes berechnen
            // und entsprechend lange schlafen:
            long jetzt = System.currentTimeMillis();

            // Den naechsten taktAnfang berechnen, der in der Zukunft
            // liegt (d.h. groesser als jetzt ist):
            do taktAnfang += DAUER_EINES_TAKTS; while (taktAnfang < jetzt);
            long schlafDauer = taktAnfang - jetzt; // Ist nie negativ!
            try {
               sleep(schlafDauer);
            } catch (InterruptedException ex) {
               printf("Ausnahme:%n");
               printf("%s%n", ex.toString());
            }
         } // while
      } // run
      // ------------------------------------------------------------------
   } // class FortschrittsDaemon
   // =====================================================================
   // Die innere Klasse DialogGeneratorLauscher
   // =====================================================================
   class DialogGeneratorLauscher implements ActionListener {
      // Ein Objekt dieser Klasse wird beim (einzigen) Objekt der Klasse
      // DialogGeneratorFenster angemeldet (mit addActionListener).
      // Jeder Aufruf der Methode actionPerformed erzeugt ein kleines
      // Dialog-Fenster, welches zwei Textzeilen (einF und ausF) enthaelt.
      // In die obere Zeile kann der Benutzer eine Ganzzahl eingeben.
      // In der unteren Zeile wird dann ein Pruefergebnis angezeigt
      // (z.B. "istPrim(1.234.567): false" oder "istPrim(17): true".

      public void actionPerformed(ActionEvent e) {

         new DialogFenster();

         // Fuer den Fall, dass spaeter noch ein weiteres Dialog-Fenster
         // erzeugt wird, werden hier schon mal seine Indizes iX und iY
         // berechnet (Indizes im Gitter aller Dialog-Fenster):
         if        (iY < maxIY-1) { // Noch Platz in der aktuellen Spalte?
            iY++;
         } else if (iX < maxIX-1) { // Gibt es noch eine weitere Spalte?
            iY = 0;
            iX++;
         } else {                   // Wenn alle Spalten und Zeilen schon
            iX = 0;                 // einmal gefuellt worden sind, fangen
            iY = 0;                 // wir wieder bei (0, 0) an
            difX += d;              // Aber um ein paar Pixel verschoben
            difY += d;              // (in x- und in y-Richtung)
         }
      } // actionPerformed
   } // class DialogGeneratorLauscher
   // =====================================================================
   // Die innere Klasse DialogFenster (und ihr "Zubehoer")
   // =====================================================================
   // Jedes neu erzeugte DialogFenster-Objekt wird von seinem Konstruktor
   // in die globale Sammlung alleDialogFenster eingefuegt, damit der
   // FortschrittsDaemon es dort finden kann:
   ArrayList<DialogFenster> alleDialogFenster = new
   ArrayList<DialogFenster>();

   class DialogFenster extends JDialog {
      Box        vBox     = Box.createVerticalBox();
      JTextField einF     = new JTextField(20);
      JTextField ausF     = new JTextField(20);

      int        dialogNr = alleDialogFenster.size() + 1;
      String     titel    = "Dialog " + dialogNr + " (Primtest)";
      // ------------------------------------------------------------------
      DialogFenster() {
         // Allgemein gilt fuer JDialog-Fenster: Sie koennen nicht
         // "allein und selbstaendig" erzeugt werden, sondern nur als
         // "Anhaengsel zu einem anderen Fenster".
         // Hier wird jedes Dialog-Fenster "als Anhaengsel" des
         // umgebenden DialogGenerator-Fensters erzeugt.

         // Allgemein gilt fuer eine innere Klasse I einer Klasse K:
         //   this bezeichnet innerhalb von I das aktuelle  I-Objekt
         // K.this bezeichnet innerhalb von I das umgebende K-Objekt

         super(DialogGeneratorFenster.this);
         DialogFenster.this.setTitle(titel);

         Font font = new Font("Monospaced", Font.BOLD, 14);

         // Ein DialogLauscher-Objekt sollte das DialogFenster-Objekt
         // "kennen", bei dem es angemeldet wird (deshalb "this"):
         einF.addActionListener(new DialogLauscher(this));
         einF.setFont(font);

         ausF.setFont(font);
         ausF.setEditable(false);
         ausF.setText("Eine Ganzzahl?");

         this.getContentPane().add(vBox, BorderLayout.CENTER);
         vBox.add(einF);
         vBox.add(ausF);

         this.setBounds(
            (difX + iX*(difSX+d)), (difY + iY*(difSY+d)), difSX, difSY);

         // Das neu erzeugte Dialog-Fenster in eine Sammlung einfuegen
         // (damit der FortschrittsDaemon es dort finden kann):
         alleDialogFenster.add(this);
         this.setVisible(true);
      } // Konstruktor DialogFenster
      // ------------------------------------------------------------------
   } // class DialogFenster
   // =====================================================================
   // Fuer 14 relativ grosse Zahlen (7 Primzahlen und die Quadrate dieser
   // Primzahlen) werden hier "Kurzwahlnummern" festgelegt. Wenn der
   // Benutzer die "Kurzwahlnummer" eigibt, wird sie durch die zugehoerige
   // grosse Zahl ersetzt.
   static final Map<Long, Long> gross = new HashMap<Long, Long>();

   static { // statischer Initialisierer fuer das Map-Objekt gross:
      gross.put( 1L,              150000449L); // Prinzahl
      gross.put( 2L,              250000139L); // Prinzahl
      gross.put( 3L,             1000000021L); // Primzahl
      gross.put( 4L,             1234567811L); // Primzahl
      gross.put( 5L,             1234567891L); // Primzahl
      gross.put( 6L,             2000000087L); // Primzahl
      gross.put( 7L,             2000000089L); // Primzahl
      gross.put(-1L,  150000449L* 150000449L); // Quadratzahl
      gross.put(-2L,  250000139L* 250000139L); // Quadratzahl
      gross.put(-3L, 1000000021L*1000000021L); // Quadratzahl
      gross.put(-4L, 1234567811L*1234567811L); // Quadratzahl
      gross.put(-5L, 1234567891L*1234567891L); // Quadratzahl
      gross.put(-6L, 2000000087L*2000000087L); // Quadratzahl
      gross.put(-7L, 2000000089L*2000000089L); // Quadratzahl
   } // statischer Initialisierer fuer das Map-Objekt

   // Die Hintergrundfarbe eines nicht-editierbaren JTextField-Objekts:
   static Color ausBackground = new Color(238, 238, 238);

   class DialogLauscher implements ActionListener {
      // Ein neues Objekt dieser Klasse wird bei der oberen Textzeile
      // von jedem Dialog-Fenster angemeldet (mit der Methode
      // addActionListener). Wenn der Benutzer dann eine Ganzzahl in die
      // Textzeile eingibt und auf Return drueckt, wird die Methode
      // actionPerformed in diesem Objekt aufgerufen, erzeugt einen
      // neuen ArbeitsFaden (thread) und startet ihn.

      DialogFenster df;
      JTextField    einF; // Zum Eingeben einer Ganzzahl
      JTextField    ausF; // Zum Ausgeben eines Pruefergebnisses

      public DialogLauscher(DialogFenster df) {
         this.df   = df;
         this.einF = df.einF;
         this.ausF = df.ausF;
      } // Konstruktor DialogLauscher

      public void actionPerformed(ActionEvent e) {
         // Ein (moeglicherweise vorhandenes) altes Ergebnis loeschen:
         ausF.setBackground(ausBackground);
         ausF.setText(fortschrittsZeichen.get(0));

         // Die (hoffentlich) vom Benutzer eingegebene Ganzzahl holen:
         long enn = unformat(einF.getText());

         // Einige kleine Zahlen 1, 2, 3, ... werden auf grosse Zahlen
         // abgebildet, bei denen der Primzahltest etwas laenger dauert.
         if (gross.containsKey(enn)) enn = gross.get(enn);

         // Die Zahl enn lesbar formatiert zurueck in die Eingabezeile
         // schreiben (falls dort eine Eingabe steht, die schwer lesbar
         // oder falsch ist (wie z.B. 1000000000000000 bzw. "ABC")
         einF.setText(format(enn));

         // Jetzt wird ein weiterer Faden erzeugt (er startet sich selbst):
         ArbeitsFaden faden = new ArbeitsFaden(df, enn);
      } // actionPerformed
   } // class DialogLauscher
   // =====================================================================
   class ArbeitsFaden extends Thread {
      // Jedes Objekt dieser Klasse prueft als Faden (thread), ob die
      // Ganzzahl enn eine Primzahl ist und zeigt das Ergebnis (true oder
      // false) im DialogFenster df an.

      DialogFenster df;  // Dieser Faden wurde von df aus gestartet
      long          enn; // Die zu testende Zahl

      public ArbeitsFaden(DialogFenster df, long enn) {
         this.df   = df;
         this.enn  = enn;
         this.start();
      } // Konstruktor ArbeitsFaden
      // ------------------------------------------------------------------
      public void run() {

         // Anfangsmeldung zur Standardausgabe:
         GregorianCalendar anf = new GregorianCalendar();
         printf(
            "Dialog %2d, T-Beg: %tT, %22s Zahl: %,d%n",
            df.dialogNr, anf, " ", enn
         );

         // Die Zahl enn testen (kann lange dauern):
         boolean ennIstPrim = istPrim(enn);

         // Das Testergebnis im DialogFenster df anzeigen:
         df.ausF.setBackground(ennIstPrim ? Color.GREEN : Color.RED);
         df.ausF.setText("istPrim(" + format(enn) + "): " + ennIstPrim);

         // Endemeldung zur Standardausgabe:
         GregorianCalendar end = new GregorianCalendar();
         long
         sek = end.getTimeInMillis() - anf.getTimeInMillis();
         sek = (sek + 999) / 1000; // Millis in Sekunden umrechnen
         printf(
            "Dialog %2d, T-End: %tT, Erg.: %5b, Sek: %3d, Zahl: %,d%n",
            df.dialogNr, end, ennIstPrim, sek, enn
         );
      } // run
      // ------------------------------------------------------------------
   } // class ArbeitsFaden
   // =====================================================================
   static String format(long enn) {
      // Wandelt enn in einen formatierten String um. Fuegt vor jeder
      // Dreiergruppe von Ziffern ein Gruppierungszeichen ein. Beispiele:
      // format(123456789012345678L) ist "123.456.789.012.345.678"
      // format(1000000000000000L)   ist   "1.000.000.000.000.000"
      StringWriter sw = new StringWriter();
      PrintWriter  pw = new PrintWriter (sw);
      pw.printf("%,d", enn);
      return sw.toString();
   } // format

   static long unformat(String str) {
      // Wandelt str in einen long-Wert um (auch wenn str Gruppierungs-
      // zeichen zwischen den Ziffern enhaelt). Beispiele:
      // unformat("1.234.567")  ist 1234567L
      // unformat("123.4.5.67") ist 1234567L
      // unformat("1234567")    ist 1234556L
      // Falls die Umwandlung nicht klappt, wird 0L als Ergebnis geliefert:
      // unformat("abc")        ist 0L
      DecimalFormat def = new DecimalFormat("##################,#");
      Number        num = def.parse(str, new ParsePosition(0));

      // def.parse liefert null, wenn etwas nicht geklappt hat:
      return num==null ? 0L : num.longValue();
   } // unformat

   // Eine Methode mit einem kurzen Namen:
   static void printf(String f, Object... v) {System.out.printf(f, v);}
   // ---------------------------------------------------------------------
} // class DialogGeneratorFenster
/* ------------------------------------------------------------------------
Eine typische Ausgabe dieses Programms zur Standardausgabe:

-----------------------------------------------------
DialogGeneratorFenster: Jetzt geht es los!

Bedienungsvorschlag:
Klicken Sie dreimal auf den Knopf 'Neuer Dialog'.
Geben Sie in jedes der drei sich oeffnenden Dialog-
Fenster eine der drei Zahlen 2, 1 und -1 ein (jeweils
in die obere Zeile) und schliessen Sie jede Eingabe
mit Return ab. Warten Sie, bis alle drei Fenster rot
bzw. gruen geworden sind. Das kann viele  Sekunden
oder sogar einige Minuten dauern.
------------------------------------------------------
'Kurzwahlnummern' fuer bestimmte Zahlen:

  Die                 Steht fuer      Ist eine
 Kurzwahl:          die grosse Zahl:  Primzahl:
    1                   150.000.449     ja
    2                   250.000.139     ja
    3                 1.000.000.021     ja
    4                 1.234.567.811     ja
    5                 1.234.567.891     ja
    6                 2.000.000.087     ja
    7                 2.000.000.089     ja
   -1        22.500.134.700.201.601     nein
   -2        62.500.069.500.019.321     nein
   -3     1.000.000.042.000.000.441     nein
   -4     1.524.157.679.957.331.721     nein
   -5     1.524.157.877.488.187.881     nein
   -6     4.000.000.348.000.007.569     nein
   -7     4.000.000.356.000.007.921     nein

Die POSITIVEN Kurzwahlnummern stehen fuer Primzahlen,
die NEGATIVEN fuer die Quadrate der Primzahlen. Z.B.
steht   -3 fuer das Quadrat der Primzahl mit der
Kurzwahl 3 .
-----------------------------------------------------
Dialog  2, T-Beg: 19:10:04,                        Zahl: 1.000.000.021
Dialog  3, T-Beg: 19:10:08,                        Zahl: 250.000.139
Dialog  1, T-Beg: 19:10:11,                        Zahl: 150.000.449
Dialog  4, T-Beg: 19:10:19,                        Zahl: 17
Dialog  4, T-End: 19:10:20, Erg.:  true, Sek:   1, Zahl: 17
Dialog  4, T-Beg: 19:10:23,                        Zahl: 12
Dialog  4, T-End: 19:10:23, Erg.: false, Sek:   0, Zahl: 12
Dialog  1, T-End: 19:10:36, Erg.:  true, Sek:  25, Zahl: 150.000.449
Dialog  3, T-End: 19:10:45, Erg.:  true, Sek:  37, Zahl: 250.000.139
Dialog  2, T-End: 19:11:21, Erg.:  true, Sek:  78, Zahl: 1.000.000.021
Dialog  1, T-Beg: 19:11:37,                        Zahl: 22.500.134.700.201.601
Dialog  1, T-End: 19:11:53, Erg.: false, Sek:  17, Zahl: 22.500.134.700.201.601
------------------------------------------------------------------------ */
