// Datei StringBuilderJut.java
/* ------------------------------------------------------------------------
JUnit ist ein Rahmenprogramm zum Erstellen und Ausfuehren von Test-
programmen. Die vorliegende Klasse StringBuilderJut wurde mit diesem
Rahmenprogramm erstellt und dient zum Testen der Standardklasse
StringBuilder. Es folgen hier ein paar wichtige, allgemeine Regeln fuer
das Erstellen einer JUnit-Klasse (JUK) fuer JUnit Version 3.8
(fuer neuere Versionen wie 4.0 und 4.2 gelten etwas andere Regeln)

1. Der Name einer JUK ist frei waehlbar (hier: StringBuilderJut).
2. Eine JUK muss die Klasse junit.framework.TestCase erweitern.
3. Eine JUK muss eine parameterlose Klassenprozedur namens suite ent-
   halten, die ein Test-Objekt liefert (junit.framework.Test ist eine
   Schnittstelle, die von der Klasse junit.framework.TestSuite implemen-
   tiert wird).
4. Die einzelnen Testfaelle (test cases) muessen als parameterlose
   Objektprozeduren ("void-Methoden") einer JUK-Klasse realisert werden.
5. Die Namen solcher Testfaelle sollten mit "test" beginnen (dann ist
   alles ganz einfach), koennen aber auch anders lauten (das macht bei 6.
   ein bisschen mehr Arbeit).
6. Die Methode suite muss die Namen aller Testfaelle zu einem TestSuite-
   Objekt zusammenfassen und dieses Objekt als Ergebnis liefern. Die
   Methode suite in dieser Klasse zeigt dafuer zwei Vorgehensweisen.

Diese JUnit-Klasse enthaelt 4 Testfaelle (namens testLeerSB, testAppend
probiereAppendInt und assertSyntax). Der Testfall assertSyntax soll nur
die Syntax von assert-Befehlen verdeutlichen, fuehrt aber keine sinnvollen
Tests durch.

Die Klasse StringBuilderJut (und damit ihre 4 Testfaelle) kann man auf
zwei Arten ausfuehren lassen:

1. Indem man das Programm StringBuilderJut aufruft (siehe die main-Methode).
2. Indem man (in einem bash- oder DOS-Fenster) das folgende Kommando eingibt:

> java junit.awtui.TestRunner StringBuilderJut

Dieses Programm setzt Junit Version 3.8 voraus und laeuft NICHT mit neueren
Version wie 4.0 oder 4.2.
------------------------------------------------------------------------ */
import junit.framework.Test;      // Eine Schnittstelle
import junit.framework.TestCase;  // Eine Test-Klasse
import junit.framework.TestSuite; // Eine Test-Klasse

public class StringBuilderJut extends TestCase {    // siehe oben 1. und 2.
   // ---------------------------------------------------------------------
   static public Test suite() {                                  // s.o. 3.
      // Ein TestSuite-Objekt ist im Wesentlichen eine Zusammenfassung
      // von Namen von Testfaellen. Ein Testfall ist eine parameterlose
      // Objektprozeudr (non-static void-method with no parameters).

      // Alle mit "test" beginnenden Namen von Testfaellen kann man
      // (pauschal) wie folgt in ein TestSuite-Objekt einfuegen:
      TestSuite ts1 = new TestSuite(StringBuilderJut.class);     // s.o. 5.

      // Beliebige Namen von Testfaellen kann man einzeln wie folgt
      // in ein TestSuite-Objekt einfuegen:
      ts1.addTest(new StringBuilderJut("probiereAppendInt"));    // s.o. 5.
      ts1.addTest(new StringBuilderJut("assertSyntax"));
      // Diese zweite Vorgehensweise setzt einen StringBuilderJut-Konstruk-
      // tor mit einem String-Parameter voraus (siehe unten).
      return ts1;                                                // s.o. 6.
   } // suite
   // ---------------------------------------------------------------------
   // Ein StringBuilderJut-Konstruktor mit einem String-Parameter:
   public StringBuilderJut(String name) {super(name);}
   // ---------------------------------------------------------------------
   // Eine globale Variable, die von allen Testfaellen benutzt werden kann:
   StringBuilder sb;

   // Die Methode setUp wird (vom JUnit-Rahmenprogramm) vor jedem
   // einzelnen Testfall aufgerufen:
   public void setUp() {
      sb = new StringBuilder();
      pln("setUp    wurde aufgerufen!");
   } // setUp

   // Die Methode tearDown wird (vom JUnit-Rahmenprogramm) nach jedem
   // einzelnen Testfall aufgerufen:
   public void tearDown() {
      pln("tearDown wurde aufgerufen!");
   }
   // ---------------------------------------------------------------------
   public void testLeerSB() {                       // 1. Testfall, s.o. 4.
      // Ist sb wirklich leer?
      assertEquals("", sb.toString());
      assertEquals( 0, sb.length());
   } // testLeerSB
   // ---------------------------------------------------------------------
   public void testAppendString() {                 // 2. Testfall, s.o. 4.
      // Funktioniert das Anhaengen eines String-Objekts mit append?
      sb.append("Hallo!");
      assertEquals("Hallo!", sb.toString());
      assertEquals(6, sb.length());
   } // testAppendString
   // ---------------------------------------------------------------------
   public void probiereAppendInt() {                // 3. Testfall, s.o. 4.
      // Funktioniert das Anhaengen eines int-Wertes mit append?
      sb.append(123);
      assertEquals("Fehler bei sb.append(123)", "123", sb.toString());
      assertEquals("Fehler bei sb.append(123)",    3 , sb.length());
   } // probiereAppendInt
   // ---------------------------------------------------------------------
   // Der folgende Testfall soll die Syntax (und Semantik) von assert-
   // Befehlen verdeutlichen, fuehrt aber keine sinnvollen Tests durch.

   public void assertSyntax() {                     // 4. Testfall, s.o. 4.
      String s1 = new String("Hallo");    // s1 und s2 sind gleich, aber
      String s2 = new String("Hallo");    // nicht identisch
      int    i1 = 17;                     // i1 und i2 sind gleich und
      int    i2 = 17;                     // identisch (primitver Typ int)
      double d1 = 17.0;                   // i1 und d1 sind ungleich
      Object o1 = null;

      assertSame   ("AB", "AB");          // Gelingt
      assertSame   (s1, s1);              // Gelingt
//    assertSame   (s1, s2);              // Misslingt
      assertEquals (s1, s2);              // Gelingt

      assertSame   (17, 17);              // Gelingt
      assertSame   (i1, i2);              // Gelingt
      assertSame   (i1, 17);              // Gelingt
      assertEquals (17, 17);              // Gelingt
      assertEquals (i1, i2);              // Gelingt
      assertEquals (i1, 17);              // Gelingt
//    assertEquals (i1, d1);              // Misslingt

      assertTrue   (true);                // Gelingt
//    assertTrue   (false);               // Misslingt
//    assertFalse  (true);                // Misslingt
      assertFalse  (false);               // Gelingt

      assertNull   (o1);                  // Gelingt
//    assertNull   (s1);                  // Misslingt

      // Vergleich von Gleitpunktzahlen mit Toleranz:
      assertEquals (10.0, 11.0,   1.0);   // Gelingt   |10.0 - 11.0  | <= 1.0
//    assertEquals (10.0, 11.001, 1.0);   // Misslingt |10.0 - 11.001| >  1.0
      assertEquals (10.0, 10.1,   0.1);   // Gelingt   |10.0 - 10.1  | <= 0.1
//    assertEquals (10.0,  9.8,   0.1);   // Misslingt |10.0 -  9.8  | >  0.1

      // Bei allen assert-Methoden kann man als ersten Parameter eine
      // Fehlermeldung (vom Typ String) angeben, die im Falle eines
      // Misslingens ausgegeben wird:
      assertSame  ("Oh weia!", i1, i2);
      assertEquals("Schlimm!", s1, s2);
      assertTrue  ("????????", true);
//    assertNull  ("Ich wusste es!", s1); // Misslingt mit Fehlermeldung!
      assertEquals("Zu ungenau!", d1, 16.9, 0.2);

      // Vergleiche mit null:
      assertSame  (null, null);           // Gelingt
//    assertSame  (s1,   null);           // Misslingt
//    assertSame  (null, s1);             // Misslingt
      assertEquals(null, null);           // Gelingt
//    assertEquals(s1,   null);           // Misslingt
//    assertEquals(null, s1);             // Misslingt
   } // assertSyntax
   // ---------------------------------------------------------------------
   static public void main(String[] _) {
      // Die 4 Testfaelle in suite() ausfuehren lassen:
      junit.awtui.TestRunner.run(StringBuilderJut.class);

      pln("Die StringBuilderJut.suite() enthaelt " +
               StringBuilderJut.suite().countTestCases() + " Testfaelle!");
      pln("StringBuilderJut: Das war's erstmal!");
   } // main
   // ---------------------------------------------------------------------
   // Eine Methode mit einem kurzen Namen:
   static void pln(Object ob) {System.out.println(ob);}
   // ---------------------------------------------------------------------
} // class StringBuilderJut
/* ------------------------------------------------------------------------
Ausgabe des Programms StringBuilderJut:

Die StringBuilderJut.suite() enthaelt 4 Testfaelle!
StringBuilderJut: Das war's erstmal!
setUp    wurde aufgerufen!
tearDown wurde aufgerufen!
setUp    wurde aufgerufen!
tearDown wurde aufgerufen!
setUp    wurde aufgerufen!
tearDown wurde aufgerufen!
setUp    wurde aufgerufen!
tearDown wurde aufgerufen!
---------------------------------------------------------------------------
assert-Methoden in Kuerze:

Es gibt zahlreiche assert-Methoden (assertEquals, assertSame, assertTrue,
assertFalse, assertNull, assertNotNull etc.).

Eine assert-Methode kann misslingen (das zaehlt als Fehler) oder gelingen.

Die Methoden namens assertEquals gelingen, wenn ihre beiden (letzten)
Parameter gleich sind. Diese Parameter duerfen zu einem primitiven Typ
(wie int, char, float, ...) oder zu einem Referenztyp (wie String,
Comparable, int[], ...) gehoeren. Wenn sie zu einem primitiven Typ
gehoeren, werden sie mit dem Operator == verglichen. Wenn sie zu einem
Referenztyp gehoeren, werden sie mit equals verglichen. Beide Parameter
sollten zum selben Typ gehoeren.

Die Methoden namens assertSame gelingen, wenn ihre (letzten) beiden
Parameter identisch sind. Diese Parameter duerfen zu einem primitiven
Typ (wie int, char, float, ...) oder zu einem Referenztyp (wie String,
Comparable, int[], ...) gehoeren. Sie werden auf jeden Fall mit dem
Operator == verglichen. Beide Parameter sollten zum selben Typ gehoeren.

Die Methode assertTrue gelingt, wenn ihr (letzter) Parameter gleich true
ist. Fuer die Methoden assertFalse und assertNull gilt Entsprechendes.

Bei allen assert-Methoden kann man als ersten Parameter eine Fehler-
meldung (vom Typ String) angeben, die ausgegeben wird, wenn die
Methode misslingt.

Die Methode assertNotSame gelingt, wenn assertSame misslingt (und
umgekehrt). Entsprechend fuer assertNotNull und assertNull. Eine Methode
assertNotEquals gibt es nicht (wieso?).

Wenn man Gleitpunktzahlen (vom Typ float bzw. double) mit assertEquals
vergleicht, muss man als letzten Parameter eine Toleranz angeben.
Nur wenn die beiden verglichenen Zahlen sich um mehr als diese Toleranz
unterscheiden, gelten sie als ungleich, sonst als gleich.

Anmerkung: Diese Toleranz ist eine absolute Zahl (d.h. unabhaengig von
der Groesse der verglichenen Zahlen). Wenn man eine relative Abweichung
zulassen will (z.B. um 3 Prozent oder um 0.05 Promille etc.), muss man
selbst eine entsprechende assert-Methode programmieren.
------------------------------------------------------------------------ */
