// Datei RandomWalk01Jut.java
/* ------------------------------------------------------------------------
Erzeugt (mehrmals) ein RandomWalk01-Objekt rw und testet die
Methoden rw.nextA, rw.nextB und rw.nextC.

Dieses Programm setzt Junit Version 3.8 voraus
(und laeuft NICHT mit neueren Version wie 4.0 oder 4.2 etc.).
------------------------------------------------------------------------ */
import junit.framework.Test;      // Eine Schnittstelle
import junit.framework.TestCase;  // Eine Test-Klasse
import junit.framework.TestSuite; // Eine Test-Klasse
import java.util.Random;

public class RandomWalk01Jut extends TestCase {
   // ---------------------------------------------------------------------
   static final boolean TST1 = true;
   static final boolean TST2 = false;
   // ---------------------------------------------------------------------
   static public Test suite() {
      return new TestSuite(RandomWalk01Jut.class);
   } // suite
   // ---------------------------------------------------------------------
   public RandomWalk01Jut(String name) {super(name);}
   // ---------------------------------------------------------------------
   // Globale Variablen, die von allen Testfaellen benutzt werden koennen:
   RandomWalk01 rw;   // Das zu testende Objekt
   UnRandomAB   urAB; // Ein Ersatz fuer ein "richtiges Random-Objekt"
   UnRandomC    urC;  // Ein Ersatz fuer ein "richtiges Random-Objekt"

   public void setUp() {
      rw = new RandomWalk01("Dieser Titel wird ignoriert!");
      // Der Parameter des hier aufgerufenen RandomWalk01-Konstruktors
      // dient nur dazu, diesen "besonderen Konstruktor" von den "normalen
      // Konstruktoren" zu unterscheiden. Der besondere Konstruktor erzeugt
      // kein PoxelPanel-Objekt, kein JFrame-Objekt und keinen Zufallsweg,
      // ermoeglicht es aber, die Methoden rw.nextA, rw.nextB und rw.nextC
      // zu testen.
   } // setUp
   // =====================================================================
   static private class UnRandomAB extends Random {
      // Den zu testenden Methoden nextA und nextB wird jeweils ein
      // UnRandomAB-Objekt urAB (als letzter Parameter) uebergeben.
      // Dadurch hat das Testprogramm die Moeglichkeit, genau zu bestimmen,
      // welchen int-Wert die Methode urAB.nextInt liefert (indem es dem
      // Attribut urAB.naechste diesen Wert zuweist).

      int naechste = Integer.MIN_VALUE;

      public int nextInt(int _) {
         return naechste;
      }
   } // class UnRandomAB
   // =====================================================================
   // Das Testen der Methode nextC ist deutlich schwieriger als
   // das Testen der Methoden nextA und nextB. Insbesondere wird fuer nextC
   // eine kompliziertere Erweiterung der Klasse Random benoetigt:

   static private class UnRandomC extends Random {
      // Den zu testenden Methode nextC wird jeweils ein
      // UnRandomC-Objekt urC (als letzter Parameter) uebergeben. Dadurch
      // hat das Testprogramm die Moeglichkeit, genau zu bestimmen,
      // welche int-Werte die Methode urC.nextInt liefert.

      int anzahl;    // anzahl der Zahlen, die von nextInt geliefert werden
      int naechste;  // naechste Zahl,     die von nextInt geliefert wird

      UnRandomC(int mDV, int erste) {
         anzahl   = 2*mDV + 1;
         naechste = erste;
      } // Konstruktor UnRandom

      public int nextInt(int _) {
         // Der Parameter wird ignoriert.
         // Liefert eine der Zahlen zwischen 0 und anzahl-1, und zwar
         // beim ersten Mal die Zahl erste und dann zyklisch immer
         // die naechste (d.h. nach anzahl-1 kommt wieder 0 etc.).

         int erg  = naechste;
         naechste = (naechste+1) % anzahl;
         return erg;
      } // nextInt

   } // class UnRandomC
   // =====================================================================
   // Die folgenden Methoden aehneln stark den gleichnamigen
   // zu testenden Methoden der Klasse RandomWalk01, erwarten aber als
   // letzten Parameter einen bestimmten Delta-Wert (und nicht ein Random-
   // Objekt, mit dem ein Delta-Wert ermittelt werden kann)

   int nextA(final int vAlt, final int mDV, final int anzV, final int dV) {
      // Rand-Strategie A: Auf der gegenueberliegenden Seite weitermachen
/*
      // Version mit if-Anweisungen:
      int vNeu = vAlt + dV;
      if (vNeu <   0  ) vNeu += anzV;
      if (vNeu >= anzV) vNeu -= anzV;
*/

      // Elegantere Version ohne if-Anweisungen:
      int vNeu = (vAlt + dV + anzV) % anzV;

      return vNeu;
   } // nextA
   // ---------------------------------------------------------------------
   int nextB(final int vAlt, final int mDV, final int anzV, final int dV) {
      // Rand-Strategie B: Am Rand spiegeln

      // Version mit if-Anweisungen:
      int vNeu = vAlt + dV;
      if (vNeu <   0  ) vNeu = -vNeu - 1;
      if (vNeu >= anzV) vNeu = anzV - (vNeu - anzV + 1);
/*
      // Elegantere Version ohne if-Anweisungen (ist vermutlich noch falsch):
      int vNeu = vAlt + dV;
      if(TST2) printf("a: vNeu: %2d, vAlt: %2d, dV: %2d%n", vNeu, vAlt, dV);
      vNeu = (2*anzV - vNeu - 1) % anzV;
      if(TST2) printf("b: vNeu: %2d, vAlt: %2d, dV: %2d%n", vNeu, vAlt, dV);
*/
      return vNeu;
   } // nextB
   // ---------------------------------------------------------------------
   int nextC(final int vAlt, final int mDV, final int anzV, final int dV) {
      // Rand-Strategie C: Einen neuen Schritt waehlen

      int vNeu = vAlt + dV;
      if (vNeu <   0  ) return Integer.MIN_VALUE;
      if (vNeu >= anzV) return Integer.MIN_VALUE;
      // Integer.MIN_VALUE bedeutet hier: Der neue Wert von vNeu liegt
      // ausserhalb des erlaubten Bereichs ("ausserhalb des PoxelPanels").

      return vNeu;
   } // nextC
   // ---------------------------------------------------------------------
   public void test_nextA_01() {
      // 7 Einzeltests

      final int vAlt = 4;
      final int mDV  = 3;
      final int anzV = 5;
      int    dV;
      String fm;
      urAB = new UnRandomAB();

      dV = 3;
      urAB.naechste = dV+mDV;
      fm = String.format("anzV:%2d, mDV:%2d, vAlt:%2d, dV:%2d%n", anzV, mDV, vAlt,dV);
      assertEquals(fm, 2, rw.nextA(vAlt, mDV, anzV, urAB));

      dV = 2;
      urAB.naechste = dV+mDV;
      fm = String.format("anzV:%2d, mDV:%2d, vAlt:%2d, dV:%2d%n", anzV, mDV, vAlt,dV);
      assertEquals(1, rw.nextA(vAlt, mDV, anzV, urAB));

      dV = 1;
      urAB.naechste = dV+mDV;
      fm = String.format("anzV:%2d, mDV:%2d, vAlt:%2d, dV:%2d%n", anzV, mDV, vAlt,dV);
      assertEquals(0, rw.nextA(vAlt, mDV, anzV, urAB));

      dV = 0;
      urAB.naechste = dV+mDV;
      fm = String.format("anzV:%2d, mDV:%2d, vAlt:%2d, dV:%2d%n", anzV, mDV, vAlt,dV);
      assertEquals(4, rw.nextA(vAlt, mDV, anzV, urAB));

      dV = -1;
      urAB.naechste = dV+mDV;
      fm = String.format("anzV:%2d, mDV:%2d, vAlt:%2d, dV:%2d%n", anzV, mDV, vAlt,dV);
      assertEquals(3, rw.nextA(vAlt, mDV, anzV, urAB));

      dV = -2;
      urAB.naechste = dV+mDV;
      fm = String.format("anzV:%2d, mDV:%2d, vAlt:%2d, dV:%2d%n", anzV, mDV, vAlt,dV);
      assertEquals(2, rw.nextA(vAlt, mDV, anzV, urAB));

      dV = -3;
      urAB.naechste = dV+mDV;
      fm = String.format("anzV:%2d, mDV:%2d, vAlt:%2d, dV:%2d%n", anzV, mDV, vAlt,dV);
      assertEquals(1, rw.nextA(vAlt, mDV, anzV, urAB));

      if (TST1) printf("test_nextA_01, anzAssert: %3d%n", 7);

   } // test_nextA_01
   // ---------------------------------------------------------------------
   public void test_nextA_02() {
      // 7 Einzeltests

      final int vAlt = 0;
      final int mDV  = 3;
      final int anzV = 5;
      int    dV;
      String fm;
      urAB = new UnRandomAB();

      dV = 3;
      urAB.naechste = dV+mDV;
      fm = String.format("anzV:%2d, mDV:%2d, vAlt:%2d, dV:%2d%n", anzV, mDV, vAlt,dV);
      assertEquals(3, rw.nextA(vAlt, mDV, anzV, urAB));

      dV = 2;
      urAB.naechste = dV+mDV;
      fm = String.format("anzV:%2d, mDV:%2d, vAlt:%2d, dV:%2d%n", anzV, mDV, vAlt,dV);
      assertEquals(2, rw.nextA(vAlt, mDV, anzV, urAB));

      dV = 1;
      urAB.naechste = dV+mDV;
      fm = String.format("anzV:%2d, mDV:%2d, vAlt:%2d, dV:%2d%n", anzV, mDV, vAlt,dV);
      assertEquals(1, rw.nextA(vAlt, mDV, anzV, urAB));

      dV = 0;
      urAB.naechste = dV+mDV;
      fm = String.format("anzV:%2d, mDV:%2d, vAlt:%2d, dV:%2d%n", anzV, mDV, vAlt,dV);
      assertEquals(0, rw.nextA(vAlt, mDV, anzV, urAB));

      dV = -1;
      urAB.naechste = dV+mDV;
      fm = String.format("anzV:%2d, mDV:%2d, vAlt:%2d, dV:%2d%n", anzV, mDV, vAlt,dV);
      assertEquals(4, rw.nextA(vAlt, mDV, anzV, urAB));

      dV = -2;
      urAB.naechste = dV+mDV;
      fm = String.format("anzV:%2d, mDV:%2d, vAlt:%2d, dV:%2d%n", anzV, mDV, vAlt,dV);
      assertEquals(3, rw.nextA(vAlt, mDV, anzV, urAB));

      dV = -3;
      urAB.naechste = dV+mDV;
      fm = String.format("anzV:%2d, mDV:%2d, vAlt:%2d, dV:%2d%n", anzV, mDV, vAlt,dV);
      assertEquals(2, rw.nextA(vAlt, mDV, anzV, urAB));

      if (TST1) printf("test_nextA_02, anzAssert: %3d%n", 7);

   } // test_nextA_02

   // ---------------------------------------------------------------------

   public void test_nextA_03() {
      // Die Methode nextA wird mit allen Kombinationen folgender
      // Paramerwerte getestet:
      // Laenge des Poxelpanels in v-Reichtung: anzV aus [5   , 9]
      // Maximales Delta        in v-Richtung:  mDV  aus [1   , 3]
      // Koordinate             in v-Richtung:  vAlt aus [0   , anzV]
      // Delta                  in v-Richtung:  dV   aus [-mDV, +mDV]

      urAB = new UnRandomAB();
      int anzAssert = 0; // Wie viele assert-Befehle werden ausgefuehrt?

      for (int anzV=5; anzV<=9; anzV++) {
         for(int mDV=1; mDV<=3; mDV++) {
            for(int vAlt=0; vAlt<anzV; vAlt++) {
               for(int dV=-mDV; dV<=+mDV; dV++) {
                  urAB.naechste = dV+mDV;
                  int soll = nextA(vAlt, mDV, anzV, dV);
                  int ist  = rw.nextA(vAlt, mDV, anzV, urAB);

                  // Eine informative Failure-Meldung zusammenbauen:
                  String fm = String.format(
                     "anzV:%2d, mDV:%2d, vAlt:%2d, dV:%2d, soll:%2d, ist:%2d%n",
                      anzV,     mDV,     vAlt,     dV,     soll,     ist
                   );

                  assertEquals(fm, soll,  ist);
                  if (TST1) anzAssert++;
               } // for dV
            } // for vAlt
         } // for mDV
      } // for anzV

      if (TST1) printf("test_nextA_03, anzAssert: %3d%n", anzAssert);

   } // test_nextA_03
   // ---------------------------------------------------------------------

   public void test_nextB_01() {
      // 7 Einzeltests

      final int vAlt = 4;
      final int mDV  = 3;
      final int anzV = 5;
      int    dV;
      String fm;
      urAB = new UnRandomAB();

      dV = 3;
      urAB.naechste = dV+mDV;
      fm = String.format("anzV:%2d, mDV:%2d, vAlt:%2d, dV:%2d%n", anzV, mDV, vAlt,dV);
      assertEquals(2, rw.nextB(vAlt, mDV, anzV, urAB));

      dV = 2;
      urAB.naechste = dV+mDV;
      fm = String.format("anzV:%2d, mDV:%2d, vAlt:%2d, dV:%2d%n", anzV, mDV, vAlt,dV);
      assertEquals(3, rw.nextB(vAlt, mDV, anzV, urAB));

      dV = 1;
      urAB.naechste = dV+mDV;
      fm = String.format("anzV:%2d, mDV:%2d, vAlt:%2d, dV:%2d%n", anzV, mDV, vAlt,dV);
      assertEquals(4, rw.nextB(vAlt, mDV, anzV, urAB));

      dV = 0;
      urAB.naechste = dV+mDV;
      fm = String.format("anzV:%2d, mDV:%2d, vAlt:%2d, dV:%2d%n", anzV, mDV, vAlt,dV);
      assertEquals(4, rw.nextB(vAlt, mDV, anzV, urAB));

      dV = -1;
      urAB.naechste = dV+mDV;
      fm = String.format("anzV:%2d, mDV:%2d, vAlt:%2d, dV:%2d%n", anzV, mDV, vAlt,dV);
      assertEquals(3, rw.nextB(vAlt, mDV, anzV, urAB));

      dV = -2;
      urAB.naechste = dV+mDV;
      fm = String.format("anzV:%2d, mDV:%2d, vAlt:%2d, dV:%2d%n", anzV, mDV, vAlt,dV);
      assertEquals(2, rw.nextB(vAlt, mDV, anzV, urAB));

      dV = -3;
      urAB.naechste = dV+mDV;
      fm = String.format("anzV:%2d, mDV:%2d, vAlt:%2d, dV:%2d%n", anzV, mDV, vAlt,dV);
      assertEquals(1, rw.nextB(vAlt, mDV, anzV, urAB));

      if (TST1) printf("test_nextB_01, anzAssert: %3d%n", 7);

   } // test_nextB_01
   // ---------------------------------------------------------------------

   public void test_nextB_02() {
      // 7 Einzeltests

      final int vAlt = 0;
      final int mDV  = 3;
      final int anzV = 5;
      int    dV;
      String fm;
      urAB = new UnRandomAB();

      dV = -3;
      urAB.naechste = dV+mDV;
      fm = String.format("anzV:%2d, mDV:%2d, vAlt:%2d, dV:%2d%n", anzV, mDV, vAlt,dV);
      assertEquals(2, rw.nextB(vAlt, mDV, anzV, urAB));

      dV = -2;
      urAB.naechste = dV+mDV;
      fm = String.format("anzV:%2d, mDV:%2d, vAlt:%2d, dV:%2d%n", anzV, mDV, vAlt,dV);
      assertEquals(1, rw.nextB(vAlt, mDV, anzV, urAB));

      dV = -1;
      urAB.naechste = dV+mDV;
      fm = String.format("anzV:%2d, mDV:%2d, vAlt:%2d, dV:%2d%n", anzV, mDV, vAlt,dV);
      assertEquals(0, rw.nextB(vAlt, mDV, anzV, urAB));

      dV = 0;
      urAB.naechste = dV+mDV;
      fm = String.format("anzV:%2d, mDV:%2d, vAlt:%2d, dV:%2d%n", anzV, mDV, vAlt,dV);
      assertEquals(0, rw.nextB(vAlt, mDV, anzV, urAB));

      dV = 1;
      urAB.naechste = dV+mDV;
      fm = String.format("anzV:%2d, mDV:%2d, vAlt:%2d, dV:%2d%n", anzV, mDV, vAlt,dV);
      assertEquals(1, rw.nextB(vAlt, mDV, anzV, urAB));

      dV = 2;
      urAB.naechste = dV+mDV;
      fm = String.format("anzV:%2d, mDV:%2d, vAlt:%2d, dV:%2d%n", anzV, mDV, vAlt,dV);
      assertEquals(2, rw.nextB(vAlt, mDV, anzV, urAB));

      dV = 3;
      urAB.naechste = dV+mDV;
      fm = String.format("anzV:%2d, mDV:%2d, vAlt:%2d, dV:%2d%n", anzV, mDV, vAlt,dV);
      assertEquals(3, rw.nextB(vAlt, mDV, anzV, urAB));

      if (TST1) printf("test_nextB_02, anzAssert: %3d%n", 7);

   } // test_nextB_02
   // ---------------------------------------------------------------------
   public void test_nextB_03() {
      // Die Methode nextB wird mit allen Kombinationen folgender
      // Paramerwerte getestet:
      // Laenge des Poxelpanels in v-Reichtung: anzV aus [5   , 9]
      // Maximales Delta        in v-Richtung:  mDV  aus [1   , 3]
      // Koordinate             in v-Richtung:  vAlt aus [0   , anzV]
      // Delta                  in v-Richtung:  dV   aus [-mDV, +mDV]

      urAB = new UnRandomAB();
      int anzAssert = 0; // Wie viele assert-Befehle werden ausgefuehrt?

      for (int anzV=5; anzV<=9; anzV++) {
         for(int mDV=1; mDV<=3; mDV++) {
            for(int vAlt=0; vAlt<anzV; vAlt++) {
               for(int dV=-mDV; dV<=+mDV; dV++) {
                  urAB.naechste = dV+mDV;
                  int soll = nextB(vAlt, mDV, anzV, dV);
                  int ist  = rw.nextB(vAlt, mDV, anzV, urAB);

                  // Eine informative Failure-Meldung zusammenbauen:
                  String fm = String.format(
                     "anzV:%2d, mDV:%2d, vAlt:%2d, dV:%2d, soll:%2d, ist:%2d%n",
                      anzV,     mDV,     vAlt,     dV,     soll,     ist
                   );

                  assertEquals(fm, soll,  ist);
                  if (TST1) anzAssert++;
               } // for dV
            } // for vAlt
         } // for mDV
      } // for anzV

      if (TST1) printf("test_nextB_03, anzAssert: %3d%n", anzAssert);

   } // test_nextB_03
   // ---------------------------------------------------------------------
   public void test_nextC() {
      // Die Methode nextB wird mit allen Kombinationen folgender
      // Paramerwerte getestet:
      // Laenge des Poxelpanels in v-Reichtung: anzV aus [5   , 9]
      // Maximales Delta        in v-Richtung:  mDV  aus [1   , 3]
      // Koordinate             in v-Richtung:  vAlt aus [0   , anzV]
      // Delta                  in v-Richtung:  dV   aus [-mDV, +mDV]

      int anzAssert = 0; // Wie viele assert-Befehle werden ausgefuehrt?

      for (int anzV=5; anzV<=9; anzV++) {
         for(int mDV=1; mDV<=3; mDV++) {
            for(int vAlt=0; vAlt<anzV; vAlt++) {
               for(int dV=-mDV; dV<=+mDV; dV++) {

                  // Um den soll-Wert zu ermitteln, wird ab dV und
                  // zyklisch (im Intervall [-mDV, +mDV]) nach dem ersten
                  // Wert dV2 gesucht, fuer den gilt: vAlt+dV2 liegt innerhalb
                  // des Intervalls [0, anzV-1] ("innerhalb des Panels").
                  // vAlt+dV2 ist dann der soll-Wert.
                  int dV2  = dV;
                  int soll;
                  while (true) {
                     soll = nextC(vAlt, mDV, anzV, dV2);
                     if (soll != Integer.MIN_VALUE) break;
                     dV2 = (dV2 < mDV) ? dV2+1 : -mDV;
                  }

                  urC = new UnRandomC(mDV, dV+mDV);
                  // Fuer die Methode urC.nextInt gilt:
                  // Beim ersten Aufruf liefert sie dV+mDV, und bei jedem
                  // weiteren Aufruf zyklisch den naechsten Wert aus
                  // dem Intervall [0, 2*mDV]
                  // Das ist notwendig/richtig, weil in der zu testenden
                  // Methode nextC (hoffentlich) folgender Ausdruck steht:
                  // ... urC.nextInt(...) - mDV ...
                  // und dann einen Wert aus dem Intervall [-mDV, +mDV] hat.
                  int ist  = rw.nextC(vAlt, mDV, anzV, urC);

                  // Eine informative Failure-Meldung zusammenbauen:
                  String fm = String.format(
                     "anzV:%2d, mDV:%2d, vAlt:%2d, dV:%2d, dV2:%2d, soll:%2d, ist:%2d%n",
                      anzV,     mDV,     vAlt,     dV,     dV2,     soll,     ist
                   );

                  assertEquals(fm, soll,  ist);
                  if (TST1) anzAssert++;
               } // for dV
            } // for vAlt
         } // for mDV
      } // for anzV

      if (TST1) printf("test_nextC,    anzAssert: %3d%n", anzAssert);

   } // test_nextC
   // ---------------------------------------------------------------------
   static public void main(String[] _) {

      junit.awtui.TestRunner.run(RandomWalk01Jut.class);

      if (TST1)
      printf("RandomWalk01Jut.suite() enthaelt %d Testfaelle!%n",
              RandomWalk01Jut.suite().countTestCases());
      printf("RandomWalk01Jut: Das war's erstmal!%n");
   } // main
   // ---------------------------------------------------------------------
   // Eine Methode mit einem kurzen Namen:
   static void printf(String f, Object... v) {System.out.printf(f, v);}
   // ---------------------------------------------------------------------
} // class RandomWalk01Jut
/* ------------------------------------------------------------------------
Versuch, die Randstrategien der Methoden nextA und nextB durch ASCII-Grafiken
zu illustrieren und anschaulich zu machen.


   Randstrategie A                          Randstrategie B
   "Auf der anderen Seite weitermachen"     "Am Rand spiegeln"

              +----+                                   +----+
    +------ 7 |    |                         +------ 7 |    |
    |         |    | 7                       |         |    | 7
    |         +----+                         |         +----+
    | +---- 6 |    |                         | +---- 6 |    |
    | |       |    | 6                       | |       |    | 6
    | |       +----+                         | |       +----+
    | | +-- 5 |    |                         | | +-- 5 |    |
    | | |     |    | 5                       | | |     |    | 5
    | | |   ==+====+==                       | | |   ==+====+==
    | | |   4 |    |                         | | +-> 4 |    |
    | | |     |    | 4 <-+                   | |       |    | 4
    | | |     +----+     |                   | |       +----+
    | | |   3 |    |     |                   | +---> 3 |    |
    | | |     |    | 3 <-|-+                 |         |    | 3
    | | |     +----+     | |                 |         +----+
    +-|-|-> 2 |    |     | |                 +-----> 2 |    |
      | |     |    | 2 <-|-|-+                         |    | 2 <-----+
      | |     +----+     | | |                         +----+         |
      +-|-> 1 |    |     | | |                       1 |    |         |
        |     |    | 1   | | |                         |    | 1 <---+ |
        |     +----+     | | |                         +----+       | |
        +-> 0 |    |     | | |                       0 |    |       | |
              |    | 0   | | |                         |    | 0 <-+ | |
            ==+====+==   | | |                       ==+====+==   | | |
          - 1 |    |     | | |                      -1 |    |     | | |
              |    |-1 --+ | |                         |    |-1 --+ | |
              +----+       | |                         +----+       | |
          - 2 |    |       | |                      -2 |    |       | |
              |    |-2 ----+ |                         |    |-2 ----+ |
              +----+         |                         +----+         |
          - 3 |    |         |                      -3 |    |         |
              |    |-3 ------+                         |    |-3 ------+
              +----+                                   +----+

Beide ASCII-Grafiken zeigen eine Spalte eines PoxelPanels, die aus 5 Poxeln
(mit den Indizes 0 bis 4) besteht. Zusaetzlichen werden jeweils dagestellt
drei weitere Poxel unter der Spalte (Indizes -1 bis -3) und
drei weitere Poxel ueber der Spalte (Indizes  5 bis  7).

Die Pfeile links und rechts neben den Poxel-Spalten sollen deutlich machen,
von welchem Poxel oberhalb bzw. unterhalb des PoxelPanels zu welchem
Poxel im Panel gesprungen wird.

Beispiel 1: Wenn bei der Randstrategie A "Auf der anderen Seite weitermachen"
ein Zufallsschritt erstmal zum Poxel -1 fuehrt, wird automatisch
(um 5 Poxel nach oben) zum Poxel 4 gesprungen. Vom Poxel 6 wird automatisch
(um 5 Poxel nach unten) zum Poxel 1 gesprungen.

Beispiel 2: Wenn bei der Randstrategie B "Am Rand spiegeln"
ein Zufallsschritt erstmal zum Poxel -1 fuehrt, wird automatisch
(um 1 Poxel nach oben) zum Poxel 0 gesprungen. Vom Poxel 6 wird automatisch
(un 3 Poxel nach unten) zum Poxel 3 gesprungen.
------------------------------------------------------------------------ */
