// Datei MatcherFloat.java
/* ------------------------------------------------------------------------
Die Methode   float Float.parseFloat(String s)   versucht, ihren Paramter s
in einen float-Wert umzuwandeln. Falls das nicht gelingt, wirft sie eine
NumberFormatException.

Die Methode MatcherFloat.isFloat liefert true, wenn ihr String-Parameter
von der Methode Float.parseFloat umgewandelt werden kann.

Die Methode MatcherFloat.isStartOfFloat liefert true, wenn
ihr String-Parameter ein Anfangsstueck eines Strings ist,
der von der Methode Float.parseFloat umgewandelt werden kann.

Die vorliegende Klasse diente dazu, die Voraussetzungen fuer diese beiden
Methoden (vor allem den regulaeren Ausdruck ParsableAsFloat) zu entwickeln
und zu testen.
---------------------------------------------------------------------------
Die hier besoders wichtige Vereinbarung des regulaeren Ausdrucks
ParsableAsFloat folgt (auch mit den Namen der Hilfsvariablen NonZeroDigit,
Digit, Digits etc.) der Grammatik fuer float-Literale im Buch
"The Java Language Specification" von
J. Gosling, B. Joy, G. Steele, G. Bracha,
Third Edition, Addison-Wesley, 2005
(siehe dort den Abschnitt 3.10.2 Floating-Point Literals).
---------------------------------------------------------------------------
Beispiele fuer Strings, die von der Methode Float.parseFloat umgewandelt
werden koennen:
"0.0", "+123.45", "-123.45e31", "+123.45E-31", "000.00",
"0x1p2", "0Xa3f7p0", "0xA1P-17",
"Infinity", "NaN"

Gegenbeispiele (bei denen die Methode Float.parseFloat eine
NumberFormatException wirft):
"0", "+123", "0x0", "0x123", "infinity", "nan", "NAN".

Anmerkung: Alle Buchstaben koennen wahlweise gross oder klein geschrieben
werden. Ausnahme: Die Worte "Infinity" und "NaN" muessen genau wie ange-
geben geschrieben werden, sonst wird eine Ausnahme geworfen.
------------------------------------------------------------------------ */
import java.util.regex.Pattern;
import java.util.regex.Matcher;

class MatcherFloat {
   // ---------------------------------------------------------------------
   static boolean isFloat(String str) {
      // Liefert true, wenn die Methode Float.parseFloat den Parameter str
      // in einen float-Wert umwandeln kann.
      // Beispiele fuer solche Strings str:
      // "2.", ".5", "2.5", "+2.5", "-2.5", "+2.5e-2", "12E2", "12f",
      // "0x1p3", "+0x1Ap9"

      // Der folgende regulaere Ausdruck rex passt auf alle Strings,
      // die von der Methode Float.parseFloat akzeptiert und in
      // float-Werte umgewandelt werden (ohne dass eine NumberFormat-
      // Exception geworfen wird). Dieser Ausdruck wurde im Rahmen der
      // Klasse MatcherFloat entwickelt und getestet.
      final String rex =
         "[+-]?((Infinity|NaN)|(([0-9]+)\\.(([0-9]+)?)" +
         "([eE]([+-]?([0-9]+)))?[fF]?|\\.([0-9]+)" +
         "([eE]([+-]?([0-9]+)))?[fF]?|([0-9]+)" +
         "([eE]([+-]?([0-9]+)))[fF]?|([0-9]+)([eE]([+-]?" +
         "([0-9]+)))?[fF]|((0[xX][0-9A-Fa-f]+\\.?|" +
         "0[xX][0-9A-Fa-f]+?\\.[0-9A-Fa-f]+)([pP]([+-]?([0-9]+)))[fF]?)))";

      final Pattern pat = Pattern.compile(rex);
      final Matcher mat = pat.matcher(str);
      return mat.matches();
   } // isFloat
   // ---------------------------------------------------------------------
   static boolean isStartOfFloat(String anf) {
      // Liefert true, wenn es einen String end gibt, so dass die Methode
      // Float.parseFloat den String anf+end in einen float-Wert umwandeln
      // kann.
      // Beispiele fuer solche Strings anf:
      // "", "+", "+2", "+2.", "+2.5", "+2.5e", "+2.5e3", "+2.5e3f",
      // "+2.5E3F", "-12", "-12.", "12F", ".12", "5e", "5e3", "65.43E21",
      // "0x12", "0x12p", "0x12p3", "+0x12p-3", "-0x12p+3", "-0X12P3"
      // "I", "Infi", "Infinity", "N", "NaN", "+Infinity", "-NaN"

      // Der folgende regulaere Ausdruck rex passt auf alle Strings,
      // die von der Methode Float.parseFloat akzeptiert und in
      // float-Werte umgewandelt werden (ohne dass eine NumberFormat-
      // Exception geworfen wird). Dieser Ausdruck wurde im Rahmen der
      // Klasse MatcherFloat erntwickelt und getestet.
      final String rex =
         "[+-]?((Infinity|NaN)|(([0-9]+)\\.(([0-9]+)?)" +
         "([eE]([+-]?([0-9]+)))?[fF]?|\\.([0-9]+)" +
         "([eE]([+-]?([0-9]+)))?[fF]?|([0-9]+)" +
         "([eE]([+-]?([0-9]+)))[fF]?|([0-9]+)([eE]([+-]?" +
         "([0-9]+)))?[fF]|((0[xX][0-9A-Fa-f]+\\.?|" +
         "0[xX][0-9A-Fa-f]+?\\.[0-9A-Fa-f]+)([pP]([+-]?([0-9]+)))[fF]?)))";

      Pattern pat    = Pattern.compile(rex);
      Matcher mat    = pat.matcher(anf);
      boolean matErg = mat.matches();
      boolean hitErg = mat.hitEnd();
      return (matErg || hitErg);
   } // isStartOfFloat
   // ---------------------------------------------------------------------
   // Zusammenbau eines Musters, welches auf alle Strings s passt,
   // aus denen die Methode Float.parseFloat(String s) float-Werte macht
   // (statt eine NumberFloatException zu werfen).
   // Das Muster wird zuerst als String namens ParsableAsFloat
   // und dann als als Pattern-Objekt   namens ParsableAsFloatP erzeugt.

   // Muster als String-Objekte (lauter Klassen-Attribute):
   static final String

   NonZeroDigit            = "[1-9]",
   Digit                   = "[0-9]",
   Digits                  = "(" + Digit + "+)",
   DecimalNumeral          = "(0|" + NonZeroDigit + Digits + "?)",

   HexDigit                = "[0-9A-Fa-f]",
   HexDigits               = HexDigit + "+",
   HexNumeral              = "0[xX]" + HexDigits,

   Sign                    = "[+-]",
   SignedInteger           = "(" + Sign + "?" + Digits + ")",

   BinaryExponentIndicator = "[pP]",
   BinaryExponent          = "(" + BinaryExponentIndicator + SignedInteger + ")",

   HexSignificand          = "(" + HexNumeral + "\\.?" + "|" +
                             "0[xX]" + HexDigits + "?\\." + HexDigits + ")",

   FloatTypeSuffix         = "[fF]", // double literals not allowed here!

   HexadecimalFloatingPointLiteral
                           = "(" + HexSignificand + BinaryExponent +
                             FloatTypeSuffix + "?)",

   ExponentIndicator       = "[eE]",
   ExponentPart            = "(" + ExponentIndicator + SignedInteger + ")",

   DFPL1                   = Digits + "\\." + "(" + Digits + "?)" +
                             ExponentPart + "?" + FloatTypeSuffix + "?",

   DFPL2                   = "\\." + Digits + ExponentPart + "?" +
                             FloatTypeSuffix + "?",

   DFPL3                   = Digits + ExponentPart +       FloatTypeSuffix + "?",
   DFPL4                   = Digits + ExponentPart + "?" + FloatTypeSuffix      ,

   DecimalFloatingPointLiteral
                           = DFPL1 + "|" + DFPL2 + "|" + DFPL3 + "|" + DFPL4,
   FloatingPointLiteral    = "(" + DecimalFloatingPointLiteral + "|" +
                                   HexadecimalFloatingPointLiteral + ")",

   NanAndInfinity          =  "(Infinity|NaN)",
   ParsableAsFloat         =  Sign + "?(" + NanAndInfinity + "|" + FloatingPointLiteral + ")";

   // Das hier relevante Muster als Pattern-Objekt:
   static final Pattern
   ParsableAsFloatP        = Pattern.compile(ParsableAsFloat);
   // ---------------------------------------------------------------------
   static final String Q = "\"";
   // ---------------------------------------------------------------------
   static int anzFehler = 0;

   static void doIt(boolean soll, Pattern pat, String str) {
      // Prueft, ob das Muster pat auf den String str passt.
      // Erhoeht anzFehler um 1, wenn das Ergebnis dieses Versuchs
      // nicht gleich soll ist.
      // Gibt ausserdem eine informative Zeile aus.

      Matcher mat = pat.matcher(str);
      boolean ist = mat.matches();
      boolean hit = mat.hitEnd();

      if (ist != soll) {
         anzFehler++;
         printf("*** ist ungleich soll:%n");
      }
      printf("soll: %-5b, ist: %-5b, hit: %-5b, str: %-15s%n", soll, ist, hit, Q+str+Q);
   } // doIt

   static void doIt(boolean soll, String str) {
      // Prueft, ob das Muster pat auf den String str passt.
      // Erhoeht anzFehler um 1, wenn das Ergebnis dieses Versuchs
      // nicht gleich soll ist.
      // Gibt ausserdem eine informative Zeile aus.

      boolean ist = isFloat(str);
      boolean hit = isStartOfFloat(str);
      if (ist != soll) {
         anzFehler++;
         printf("*** ist ungleich soll:%n");
      }
      printf("soll: %-5b, ist: %-5b, hit: %-5b, str: %-15s%n", soll, ist, hit, Q+str+Q);
   } // doIt
   // ---------------------------------------------------------------------
   static void doItIncreasingly(Pattern pat, String str) {
      // Prueft fuer jedes Anfangsstueck sub von str, ob das Muster pat
      // auf sub passt und ob dabei das Ende von sub erreicht wird
      // oder schon vor Erreichen des Endes feststeht, dass pat nicht
      // auf sub passt.
      // Erhoeht anzFehler um 1, wenn weder pat auf sub passt noch
      // das Ende von sub erreicht wurde.
      // Gibt ausserdem eine informative Zeile aus.

      for (int i=0; i<=str.length(); i++) {
         String  sub      = str.substring(0, i);
         Matcher mat      = pat.matcher(sub);
         boolean ist      = mat.matches();
         boolean hit      = mat.hitEnd();

         if (!ist && !hit) {
            anzFehler++;
            printf("*** weder ist noch hit:%n");
         }
         printf("ist: %-5b, hit: %-5b, str: %-15s, sub: %s%n",
            ist, hit, Q+str+Q, Q+sub+Q
         );
      }
   } // doItIncreasingly
   // ---------------------------------------------------------------------

   // ---------------------------------------------------------------------
   static public void main(String[] _) {
      printf("MatcherFloat: Jetzt geht es los!%n");
      // ---------------------------------------------------------------------
      // Das Muster ParsableAsFloatP (siehe oben) und einige seiner
      // "Teil-Muster" (z.B. SignedIntegerP, siehe unten) werden hier getestet:

      Pattern DecimalFloatingPointLiteralP     = Pattern.compile(DecimalFloatingPointLiteral);
      Pattern HexadecimalFloatingPointLiteralP = Pattern.compile(HexadecimalFloatingPointLiteral);
      Pattern SignedIntegerP                   = Pattern.compile(SignedInteger);

      printf("%nSignedIntegerP: %s%n",  SignedIntegerP);
      doIt(true , SignedIntegerP, "0"   );
      doIt(true , SignedIntegerP, "123" );
      doIt(true , SignedIntegerP, "+0"  );
      doIt(true , SignedIntegerP, "+123");
      doIt(true , SignedIntegerP, "-0"  );
      doIt(true , SignedIntegerP, "-123");
      doIt(true , SignedIntegerP, "000" );
      doIt(true , SignedIntegerP, "+000");
      doIt(true , SignedIntegerP, "-000");

      printf("%nHexadecimalFloatingPointLiteralP: %s%n",  HexadecimalFloatingPointLiteralP);
      doIt(true , HexadecimalFloatingPointLiteralP, "0x0p0"       );
      doIt(true , HexadecimalFloatingPointLiteralP, "0x0p12"      );
      doIt(true , HexadecimalFloatingPointLiteralP, "0X12AB34p+99");
      doIt(true , HexadecimalFloatingPointLiteralP, "0xABCDEFp-99");
      doIt(true , HexadecimalFloatingPointLiteralP, "0xABCDEFp99F");

      doIt(false, HexadecimalFloatingPointLiteralP, "-0x0p0"      );
      doIt(false, HexadecimalFloatingPointLiteralP, "+0x0p0"      );
      doIt(false, HexadecimalFloatingPointLiteralP, "x0p0"        );
      doIt(false, HexadecimalFloatingPointLiteralP, "0xp0"        );
      doIt(false, HexadecimalFloatingPointLiteralP, "0x0p"        );

      printf("%nParsableAsFloatP: %s%n", ParsableAsFloatP);
      doIt(true , "0x0p0"       );
      doIt(true , "0x0p12"      );
      doIt(true , "0X12AB34p+99");
      doIt(true , "0xABCDEFp-99");
      doIt(true , "0xABCDEFp99F");
      doIt(true, "-0x0p0"       ); // Kein HexadecimalFloatingPointLiteral, aber ParsableAsFloat
      doIt(true, "+0x0p0"       ); // Kein HexadecimalFloatingPointLiteral, aber ParsableAsFloat

      doIt(false, "x0p0"        );
      doIt(false, "0xp0"        );
      doIt(false, "0x0p"        );

      printf("%nDecimalFloatingPointLiteralP: %s%n",  DecimalFloatingPointLiteralP);
      doIt(true , DecimalFloatingPointLiteralP, "0."     );
      doIt(true , DecimalFloatingPointLiteralP,  ".0"    );
      doIt(true , DecimalFloatingPointLiteralP, "0.0"    );
      doIt(true , DecimalFloatingPointLiteralP, "000.000");
      doIt(true , DecimalFloatingPointLiteralP, "123e00" );
      doIt(true , DecimalFloatingPointLiteralP, ".123e0f");
      doIt(true , DecimalFloatingPointLiteralP, ".123E0F");
      doIt(true , DecimalFloatingPointLiteralP, "123e0"  );
      doIt(true , DecimalFloatingPointLiteralP, "123e0f" );
      doIt(true , DecimalFloatingPointLiteralP, "123e+0f");
      doIt(true , DecimalFloatingPointLiteralP, "123e-0f");
      doIt(true , DecimalFloatingPointLiteralP, "123f"   );

      doIt(false, DecimalFloatingPointLiteralP, "-2.5F"  );
      doIt(false ,DecimalFloatingPointLiteralP, "123ff"  );
      doIt(false, DecimalFloatingPointLiteralP, "123"    );
      doIt(false, DecimalFloatingPointLiteralP, ".e0"    );

      printf("%nParsableAsFloatP: %s%n",  ParsableAsFloatP);
      doIt(true , "0."     );
      doIt(true ,  ".0"    );
      doIt(true , "0.0"    );
      doIt(true , "000.000");
      doIt(true , "123e00" );
      doIt(true , ".123e0f");
      doIt(true , ".123E0F");
      doIt(true , "123e0"  );
      doIt(true , "123e0f" );
      doIt(true , "123e+0f");
      doIt(true , "123e-0f");
      doIt(true , "123f"   );
      doIt(true, "-2.5F"   ); // Kein DecimalFloatingPointLiteral, aber ParsableAsFloat

      doIt(false, "123ff"  );
      doIt(false, "123"    );
      doIt(false, ".e0"    );


      printf("%nParsableAsFloatP: %s%n",  ParsableAsFloatP);
      doIt(true , ParsableAsFloatP, "Infinity"    );
      doIt(true , ParsableAsFloatP, "+Infinity"   );
      doIt(true , ParsableAsFloatP, "-Infinity"   );
      doIt(true , ParsableAsFloatP, "NaN"         );
      doIt(true , ParsableAsFloatP, "+NaN"        );
      doIt(true , ParsableAsFloatP, "-NaN"        );
      doIt(true , ParsableAsFloatP, "0."          );
      doIt(true , ParsableAsFloatP, ".0"          );
      doIt(true , ParsableAsFloatP, "0.0"         );
      doIt(true , ParsableAsFloatP, "+0."         );
      doIt(true , ParsableAsFloatP, "-.0"         );
      doIt(true , ParsableAsFloatP, "+0.0"        );
      doIt(true , ParsableAsFloatP, "-000.00e000f");
      doIt(true , ParsableAsFloatP, "+000.00E000F");
      doIt(true , ParsableAsFloatP, "0123.45E678F");
      doIt(true , ParsableAsFloatP, "9999E0"      );
      doIt(true , ParsableAsFloatP, "9999F"       );
      doIt(true , ParsableAsFloatP, "9999."       );
      doIt(true , ParsableAsFloatP, "9999.e0f"    );

      doIt(false, ParsableAsFloatP, "infinity"    );
      doIt(false, ParsableAsFloatP, "+infinity"   );
      doIt(false, ParsableAsFloatP, "-infinity"   );
      doIt(false, ParsableAsFloatP, "nan"         );
      doIt(false, ParsableAsFloatP, "NAN"         );
      doIt(false, ParsableAsFloatP, "0"           );
      doIt(false, ParsableAsFloatP, "-1234567890" );

      printf("%nParsableAsFloatP: %s%n",  ParsableAsFloatP);
      doIt(true , "Infinity"    );
      doIt(true , "+Infinity"   );
      doIt(true , "-Infinity"   );
      doIt(true , "NaN"         );
      doIt(true , "+NaN"        );
      doIt(true , "-NaN"        );
      doIt(true , "0."          );
      doIt(true , ".0"          );
      doIt(true , "0.0"         );
      doIt(true , "+0."         );
      doIt(true , "-.0"         );
      doIt(true , "+0.0"        );
      doIt(true , "-000.00e000f");
      doIt(true , "+000.00E000F");
      doIt(true , "0123.45E678F");
      doIt(true , "9999E0"      );
      doIt(true , "9999F"       );
      doIt(true , "9999."       );
      doIt(true , "9999.e0f"    );

      doIt(false, "infinity"    );
      doIt(false, "+infinity"   );
      doIt(false, "-infinity"   );
      doIt(false, "nan"         );
      doIt(false, "NAN"         );
      doIt(false, "0"           );
      doIt(false, "-1234567890" );


      printf("%nParsableAsFloatP: %s%n", ParsableAsFloatP);
      doItIncreasingly(ParsableAsFloatP, "+00123.45e-012F");
      doItIncreasingly(ParsableAsFloatP, "-0xFFFFp9F");
      doItIncreasingly(ParsableAsFloatP, "+Infinity");
      doItIncreasingly(ParsableAsFloatP, "-NaN");

      printf("--------------------------------------------------------%n");
      printf("Anzahl Fehler insgesamt: %d%n", anzFehler);
      printf("--------------------------------------------------------%n");
      printf("MatcherFloat: Das war's erstmal!%n");
   } // main
   // ---------------------------------------------------------------------
   // Eine Methode mit einem kurzen Namen:
   static void printf(String f, Object... v) {System.out.printf(f, v);}
   // ---------------------------------------------------------------------
} // class MatcherFloat
/* ------------------------------------------------------------------------
Ausgabe:

MatcherFloat: Jetzt geht es los!

SignedIntegerP: ([+-]?([0-9]+))
soll: true , ist: true , hit: true , str: "0"
soll: true , ist: true , hit: true , str: "123"
soll: true , ist: true , hit: true , str: "+0"
soll: true , ist: true , hit: true , str: "+123"
soll: true , ist: true , hit: true , str: "-0"
soll: true , ist: true , hit: true , str: "-123"
soll: true , ist: true , hit: true , str: "000"
soll: true , ist: true , hit: true , str: "+000"
soll: true , ist: true , hit: true , str: "-000"

HexadecimalFloatingPointLiteralP: ((0[xX][0-9A-Fa-f]+\.?|0[xX][0-9A-Fa-f]+?\.[0-9A-Fa-f]+)([pP]([+-]?([0-9]+)))[fF]?)
soll: true , ist: true , hit: true , str: "0x0p0"
soll: true , ist: true , hit: true , str: "0x0p12"
soll: true , ist: true , hit: true , str: "0X12AB34p+99"
soll: true , ist: true , hit: true , str: "0xABCDEFp-99"
soll: true , ist: true , hit: false, str: "0xABCDEFp99F"
soll: false, ist: false, hit: false, str: "-0x0p0"
soll: false, ist: false, hit: false, str: "+0x0p0"
soll: false, ist: false, hit: false, str: "x0p0"
soll: false, ist: false, hit: false, str: "0xp0"
soll: false, ist: false, hit: true , str: "0x0p"

ParsableAsFloatP: [+-]?((Infinity|NaN)|(([0-9]+)\.(([0-9]+)?)([eE]([+-]?([0-9]+)))?[fF]?|\.([0-9]+)([eE]([+-]?([0-9]+)))?[fF]?|([0-9]+)([eE]([+-]?([0-9]+)))[fF]?|([0-9]+)([eE]([+-]?([0-9]+)))?[fF]|((0[xX][0-9A-Fa-f]+\.?|0[xX][0-9A-Fa-f]+?\.[0-9A-Fa-f]+)([pP]([+-]?([0-9]+)))[fF]?)))
soll: true , ist: true , hit: true , str: "0x0p0"
soll: true , ist: true , hit: true , str: "0x0p12"
soll: true , ist: true , hit: true , str: "0X12AB34p+99"
soll: true , ist: true , hit: true , str: "0xABCDEFp-99"
soll: true , ist: true , hit: true , str: "0xABCDEFp99F"
soll: true , ist: true , hit: true , str: "-0x0p0"
soll: true , ist: true , hit: true , str: "+0x0p0"
soll: false, ist: false, hit: false, str: "x0p0"
soll: false, ist: false, hit: false, str: "0xp0"
soll: false, ist: false, hit: true , str: "0x0p"

DecimalFloatingPointLiteralP: ([0-9]+)\.(([0-9]+)?)([eE]([+-]?([0-9]+)))?[fF]?|\.([0-9]+)([eE]([+-]?([0-9]+)))?[fF]?|([0-9]+)([eE]([+-]?([0-9]+)))[fF]?|([0-9]+)([eE]([+-]?([0-9]+)))?[fF]
soll: true , ist: true , hit: true , str: "0."
soll: true , ist: true , hit: true , str: ".0"
soll: true , ist: true , hit: true , str: "0.0"
soll: true , ist: true , hit: true , str: "000.000"
soll: true , ist: true , hit: true , str: "123e00"
soll: true , ist: true , hit: false, str: ".123e0f"
soll: true , ist: true , hit: false, str: ".123E0F"
soll: true , ist: true , hit: true , str: "123e0"
soll: true , ist: true , hit: false, str: "123e0f"
soll: true , ist: true , hit: false, str: "123e+0f"
soll: true , ist: true , hit: false, str: "123e-0f"
soll: true , ist: true , hit: false, str: "123f"
soll: false, ist: false, hit: false, str: "-2.5F"
soll: false, ist: false, hit: false, str: "123ff"
soll: false, ist: false, hit: true , str: "123"
soll: false, ist: false, hit: false, str: ".e0"

ParsableAsFloatP: [+-]?((Infinity|NaN)|(([0-9]+)\.(([0-9]+)?)([eE]([+-]?([0-9]+)))?[fF]?|\.([0-9]+)([eE]([+-]?([0-9]+)))?[fF]?|([0-9]+)([eE]([+-]?([0-9]+)))[fF]?|([0-9]+)([eE]([+-]?([0-9]+)))?[fF]|((0[xX][0-9A-Fa-f]+\.?|0[xX][0-9A-Fa-f]+?\.[0-9A-Fa-f]+)([pP]([+-]?([0-9]+)))[fF]?)))
soll: true , ist: true , hit: true , str: "0."
soll: true , ist: true , hit: true , str: ".0"
soll: true , ist: true , hit: true , str: "0.0"
soll: true , ist: true , hit: true , str: "000.000"
soll: true , ist: true , hit: true , str: "123e00"
soll: true , ist: true , hit: true , str: ".123e0f"
soll: true , ist: true , hit: true , str: ".123E0F"
soll: true , ist: true , hit: true , str: "123e0"
soll: true , ist: true , hit: true , str: "123e0f"
soll: true , ist: true , hit: true , str: "123e+0f"
soll: true , ist: true , hit: true , str: "123e-0f"
soll: true , ist: true , hit: true , str: "123f"
soll: true , ist: true , hit: true , str: "-2.5F"
soll: false, ist: false, hit: false, str: "123ff"
soll: false, ist: false, hit: true , str: "123"
soll: false, ist: false, hit: false, str: ".e0"

ParsableAsFloatP: [+-]?((Infinity|NaN)|(([0-9]+)\.(([0-9]+)?)([eE]([+-]?([0-9]+)))?[fF]?|\.([0-9]+)([eE]([+-]?([0-9]+)))?[fF]?|([0-9]+)([eE]([+-]?([0-9]+)))[fF]?|([0-9]+)([eE]([+-]?([0-9]+)))?[fF]|((0[xX][0-9A-Fa-f]+\.?|0[xX][0-9A-Fa-f]+?\.[0-9A-Fa-f]+)([pP]([+-]?([0-9]+)))[fF]?)))
soll: true , ist: true , hit: false, str: "Infinity"
soll: true , ist: true , hit: false, str: "+Infinity"
soll: true , ist: true , hit: false, str: "-Infinity"
soll: true , ist: true , hit: false, str: "NaN"
soll: true , ist: true , hit: false, str: "+NaN"
soll: true , ist: true , hit: false, str: "-NaN"
soll: true , ist: true , hit: true , str: "0."
soll: true , ist: true , hit: true , str: ".0"
soll: true , ist: true , hit: true , str: "0.0"
soll: true , ist: true , hit: true , str: "+0."
soll: true , ist: true , hit: true , str: "-.0"
soll: true , ist: true , hit: true , str: "+0.0"
soll: true , ist: true , hit: false, str: "-000.00e000f"
soll: true , ist: true , hit: false, str: "+000.00E000F"
soll: true , ist: true , hit: false, str: "0123.45E678F"
soll: true , ist: true , hit: true , str: "9999E0"
soll: true , ist: true , hit: false, str: "9999F"
soll: true , ist: true , hit: true , str: "9999."
soll: true , ist: true , hit: false, str: "9999.e0f"
soll: false, ist: false, hit: false, str: "infinity"
soll: false, ist: false, hit: false, str: "+infinity"
soll: false, ist: false, hit: false, str: "-infinity"
soll: false, ist: false, hit: false, str: "nan"
soll: false, ist: false, hit: false, str: "NAN"
soll: false, ist: false, hit: true , str: "0"
soll: false, ist: false, hit: true , str: "-1234567890"

ParsableAsFloatP: [+-]?((Infinity|NaN)|(([0-9]+)\.(([0-9]+)?)([eE]([+-]?([0-9]+)))?[fF]?|\.([0-9]+)([eE]([+-]?([0-9]+)))?[fF]?|([0-9]+)([eE]([+-]?([0-9]+)))[fF]?|([0-9]+)([eE]([+-]?([0-9]+)))?[fF]|((0[xX][0-9A-Fa-f]+\.?|0[xX][0-9A-Fa-f]+?\.[0-9A-Fa-f]+)([pP]([+-]?([0-9]+)))[fF]?)))
soll: true , ist: true , hit: true , str: "Infinity"
soll: true , ist: true , hit: true , str: "+Infinity"
soll: true , ist: true , hit: true , str: "-Infinity"
soll: true , ist: true , hit: true , str: "NaN"
soll: true , ist: true , hit: true , str: "+NaN"
soll: true , ist: true , hit: true , str: "-NaN"
soll: true , ist: true , hit: true , str: "0."
soll: true , ist: true , hit: true , str: ".0"
soll: true , ist: true , hit: true , str: "0.0"
soll: true , ist: true , hit: true , str: "+0."
soll: true , ist: true , hit: true , str: "-.0"
soll: true , ist: true , hit: true , str: "+0.0"
soll: true , ist: true , hit: true , str: "-000.00e000f"
soll: true , ist: true , hit: true , str: "+000.00E000F"
soll: true , ist: true , hit: true , str: "0123.45E678F"
soll: true , ist: true , hit: true , str: "9999E0"
soll: true , ist: true , hit: true , str: "9999F"
soll: true , ist: true , hit: true , str: "9999."
soll: true , ist: true , hit: true , str: "9999.e0f"
soll: false, ist: false, hit: false, str: "infinity"
soll: false, ist: false, hit: false, str: "+infinity"
soll: false, ist: false, hit: false, str: "-infinity"
soll: false, ist: false, hit: false, str: "nan"
soll: false, ist: false, hit: false, str: "NAN"
soll: false, ist: false, hit: true , str: "0"
soll: false, ist: false, hit: true , str: "-1234567890"

ParsableAsFloatP: [+-]?((Infinity|NaN)|(([0-9]+)\.(([0-9]+)?)([eE]([+-]?([0-9]+)))?[fF]?|\.([0-9]+)([eE]([+-]?([0-9]+)))?[fF]?|([0-9]+)([eE]([+-]?([0-9]+)))[fF]?|([0-9]+)([eE]([+-]?([0-9]+)))?[fF]|((0[xX][0-9A-Fa-f]+\.?|0[xX][0-9A-Fa-f]+?\.[0-9A-Fa-f]+)([pP]([+-]?([0-9]+)))[fF]?)))
ist: false, hit: true , str: "+00123.45e-012F", sub: ""
ist: false, hit: true , str: "+00123.45e-012F", sub: "+"
ist: false, hit: true , str: "+00123.45e-012F", sub: "+0"
ist: false, hit: true , str: "+00123.45e-012F", sub: "+00"
ist: false, hit: true , str: "+00123.45e-012F", sub: "+001"
ist: false, hit: true , str: "+00123.45e-012F", sub: "+0012"
ist: false, hit: true , str: "+00123.45e-012F", sub: "+00123"
ist: true , hit: true , str: "+00123.45e-012F", sub: "+00123."
ist: true , hit: true , str: "+00123.45e-012F", sub: "+00123.4"
ist: true , hit: true , str: "+00123.45e-012F", sub: "+00123.45"
ist: false, hit: true , str: "+00123.45e-012F", sub: "+00123.45e"
ist: false, hit: true , str: "+00123.45e-012F", sub: "+00123.45e-"
ist: true , hit: true , str: "+00123.45e-012F", sub: "+00123.45e-0"
ist: true , hit: true , str: "+00123.45e-012F", sub: "+00123.45e-01"
ist: true , hit: true , str: "+00123.45e-012F", sub: "+00123.45e-012"
ist: true , hit: false, str: "+00123.45e-012F", sub: "+00123.45e-012F"
ist: false, hit: true , str: "-0xFFFFp9F"   , sub: ""
ist: false, hit: true , str: "-0xFFFFp9F"   , sub: "-"
ist: false, hit: true , str: "-0xFFFFp9F"   , sub: "-0"
ist: false, hit: true , str: "-0xFFFFp9F"   , sub: "-0x"
ist: false, hit: true , str: "-0xFFFFp9F"   , sub: "-0xF"
ist: false, hit: true , str: "-0xFFFFp9F"   , sub: "-0xFF"
ist: false, hit: true , str: "-0xFFFFp9F"   , sub: "-0xFFF"
ist: false, hit: true , str: "-0xFFFFp9F"   , sub: "-0xFFFF"
ist: false, hit: true , str: "-0xFFFFp9F"   , sub: "-0xFFFFp"
ist: true , hit: true , str: "-0xFFFFp9F"   , sub: "-0xFFFFp9"
ist: true , hit: false, str: "-0xFFFFp9F"   , sub: "-0xFFFFp9F"
ist: false, hit: true , str: "+Infinity"    , sub: ""
ist: false, hit: true , str: "+Infinity"    , sub: "+"
ist: false, hit: true , str: "+Infinity"    , sub: "+I"
ist: false, hit: true , str: "+Infinity"    , sub: "+In"
ist: false, hit: true , str: "+Infinity"    , sub: "+Inf"
ist: false, hit: true , str: "+Infinity"    , sub: "+Infi"
ist: false, hit: true , str: "+Infinity"    , sub: "+Infin"
ist: false, hit: true , str: "+Infinity"    , sub: "+Infini"
ist: false, hit: true , str: "+Infinity"    , sub: "+Infinit"
ist: true , hit: false, str: "+Infinity"    , sub: "+Infinity"
ist: false, hit: true , str: "-NaN"         , sub: ""
ist: false, hit: true , str: "-NaN"         , sub: "-"
ist: false, hit: true , str: "-NaN"         , sub: "-N"
ist: false, hit: true , str: "-NaN"         , sub: "-Na"
ist: true , hit: false, str: "-NaN"         , sub: "-NaN"
--------------------------------------------------------
Anzahl Fehler insgesamt: 0
--------------------------------------------------------
MatcherFloat: Das war's erstmal!
------------------------------------------------------------------------ */