// File RechnerControl.java
/* ------------------------------------------------------------------------
This is the control component of the Rechner.

The Rechner is a simple calculator. Its structure follows the MVC pattern
(Model, View, Control). The model component determines the type of the
numbers, which are processed. The view component is the graphical user
interface (GUI) of the Rechner.
The model must implement the interface RechnerModel_I and
the view  must implement the interface RechnerView_I.

Authors: Agathe Merceron, Ulrich Grude
------------------------------------------------------------------------ */
package rechner;

import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.event.KeyListener;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import javax.swing.JTextField;

public class RechnerControl {

   static boolean TST = false; // To activate additional test outputs

   private RechnerModel_I aModel;
   private RechnerView_I  aView;
   // ---------------------------------------------------------------------
   public RechnerControl(RechnerModel_I aModel, RechnerView_I aView) {
      this.aModel = aModel;
      this.aView  = aView;
      // 1 listener for the submitButton in the view:
      ActionListener mal  = new MyActionListener();

      // 2 listeners for the input text fields ind the view:
      KeyListener    mkl1 = new MyKeyListener();
      KeyListener    mkl2 = new MyKeyListener();

      // Register the 3 listeners with the view:
      aView.register(mal, mkl1, mkl2);
   } // Konstruktor
   // ---------------------------------------------------------------------
   // The following listener classes for GUI elements are declared here
   // (and not in the view), because they call methods which are visible
   // here, but not in the view (calculate and isValidEntry)
   //
   // ActionListener for the submitButton in the view:
   private class MyActionListener  implements ActionListener {
      public void actionPerformed(ActionEvent e) {
         String   s1  = aView.getNumber1();
         String   s2  = aView.getNumber2();
         String[] res = aModel.calculate(s1, s2);
         aView.setResults(res);
      }
   }

   // KeyListener for the input text fields in the view:
   private class MyKeyListener  extends KeyAdapter {
      String oldContent  = "";

      public void keyReleased(KeyEvent ke) {
         JTextField tf          = (JTextField) ke.getSource();
         String     newContent  = tf.getText();
         int        newCaretPos = tf.getCaretPosition();
         boolean    isValid     = aModel.isValidEntry(newContent);

         if (TST) printf("keyReleased: newCont: %-10s, newPos: %2d%n",
            newContent, newCaretPos);

         if (isValid) {
            oldContent  = newContent;
         } else {
            // If the newContent is not valid, this is probably due to
            // the character typed last and the following holds:
            // 1. The caret (cursor) is positioned immediately to
            //    the right of the faulty character.
            // 2. This character is not present in the oldContent.
            // 3. The caret position in the oldContent was probably
            //    one less than it is in the newContent.
            // If the user used "copy and paste" (instead of typing)
            // things might be different.

            // Restore the oldContent and the old caret position:
            tf.setText         (oldContent);
            tf.setCaretPosition(Math.max(0, newCaretPos-1));
         }
      } // keyReleased
   } // class MyKeyAdapter
   // ---------------------------------------------------------------------
   // A method with a short name:
   static void printf(String f, Object... v) {System.out.printf(f, v);}
   // ---------------------------------------------------------------------
} // class RechnerControl