/* ------------------------------------------------------------------------
Model components of the Rechner should implement this interface.
See file RechnerControl.java for more information about the Rechner.

Authors: Agathe Merceron, Ulrich Grude
------------------------------------------------------------------------ */
package rechner;

public interface RechnerModel_I {

  String getTypeName();
  // Each model (class) implementing this interface should
  // "work and calculate" with numbers of a certain type,
  // e.g. of type int or of type float or ... .
  // This method returns the name of this type as a String
  // e.g. "int" or "float" or ... .

  boolean isValidEntry(String pre);
  // Returns true, iff there is a String suf, such that
  // the String pre+suf can be interpreted as a number
  // of the type expected by the implementing model.

  String[] calculate(String number1, String number2);
  // Trys to interpret number1 and number2 as numbers z1 and z2
  // (e.g. of type int or float or ... ), calculates the 5 values
  // z1+z2, z1-z2, z1*z2, z1/z2, z1%z2 and returns them as an
  // array of 5 String-objects. In case of problems the String-
  // objects may contain some error messages.

} // inteface RechnerModel_I
