// Datei GenParams01.java
/* ------------------------------------------------------------------------
Dieses Programm GenParams01 demonstriert generische (Objekt-) Methoden
mit unbegrenzten und begrenzten Typ-Parametern (generischen Parametern).
------------------------------------------------------------------------ */
import java.util.List;
import java.util.Arrays;
import java.util.ArrayList;
import java.util.Comparator;
import java.io.Serializable;
import java.rmi.Remote;

class GenParams01 {
   // ---------------------------------------------------------------------
   // Generische Methode mit einem *unbeschraenkten* Typ-Parameter:
   <T> void pA(T t) {pln("pA: " + t);}

   // Generische Methoden mit ein bis zwei *beschraenkten* Typ-Parametern:
   <T extends Number>
      void pB(T t) {pln("pB: " + t);}

   <T extends Number & Serializable & Remote>
      void pC(T t) {pln("pC: " + t);};

   <K extends Number, S extends ArrayList<K>>
      void pD(S s) {pln("pD: " + s);};

   <S extends ArrayList<?>>
      void pE(S s) {pln("pE: " + s);};

   <C extends Comparable<C>>
      void pF(C ob) {pln("pF: " + ob);};

   <T extends Comparable<?>>
      void pG(T t) {pln("pG: " + t);};

   <T extends Comparable<? extends T>>
      void pH(T t) {pln("pH: " + t);};

   <T extends Comparable<? super T>>
      void pI(T t) {pln("pI: " + t);}

   // Ein unbeschraenkter Typ-Parameter <T>, aber beschraenkte Paramtertypen:
   <T> void pJ(List<T> lst, Comparator<? super T> ctor) {pln("pI: " + lst);};
   // ---------------------------------------------------------------------
   static class IKA extends Number implements Serializable, Remote {
      public byte     byteValue() {return 0   ;}
      public double doubleValue() {return 0.0D;}
      public float   floatValue() {return 0.0F;}
      public int       intValue() {return 0   ;}
      public long     longValue() {return 0L  ;}
      public short   shortValue() {return 0   ;}

      public String toString() {return "IKA 0";}
   }
   // ---------------------------------------------------------------------
   static class IKB implements Comparable<String> {
      public int    compareTo(String s) {return -1;}
      public String toString ()         {return "IKB";}
   }
   // ---------------------------------------------------------------------
   static class IKC implements Comparable<IKD> {
      int wert;
      public        IKC      (int wert) {this.wert = wert;}
      public int    compareTo(IKD that) {return this.wert - that.wert;}
      public String toString ()         {return "IKC " + wert;}
   }
   // ---------------------------------------------------------------------
   static class IKD extends IKC {
      public        IKD      (int wert) {super(wert);}
      public String toString ()         {return "IKD " + wert;}
   }
   // ---------------------------------------------------------------------
   static class IKE implements Comparable<IKE> {
      int wert;
      public        IKE      (int wert) {this.wert = wert;}
      public int    compareTo(IKE that) {return this.wert - that.wert;}
      public String toString ()         {return "IKE " + wert;}
   }
   // ---------------------------------------------------------------------
   static class IKF extends IKE {
      public        IKF      (int wert) {super(wert);}
      public String toString ()         {return "IKF " + wert;}
   }
   // ---------------------------------------------------------------------
   static class IKG implements Comparator<String> {
      public int compare(String s1, String s2) {
         return -s1.compareTo(s2);
      }
   }
   // ---------------------------------------------------------------------
   static class IKH implements Comparator<Object> {
      public int compare(Object ob1, Object ob2) {
         return -(ob1.toString().compareTo(ob2.toString()));
      }
   }
   // ---------------------------------------------------------------------
   static public void main(String[] _) {

      // Weil pA, pB, ..., pI Objektmethoden sind, brauchen wir ein Objekt:
      GenParams01 ob = new GenParams01();

      printf("GenParams01: Jetzt geht es los!%n");
      printf("-------------------------------%n");
      ob.pA(new StringBuilder("Hallo pA!"));
      ob.pA(new Double(123.4));
      ob.pA(234.5);
      ob.pA(new ArrayList<String>());
      printf("-------------------------------%n");
      ob.pB(new Double(2.5));
      ob.pB(new Long  (123));
      printf("-------------------------------%n");
      ob.pC(new IKA());
      printf("-------------------------------%n");
      ob.pD(new ArrayList<Number>());
      ob.pD(new ArrayList<Long>());
      printf("-------------------------------%n");
      ob.pE(new ArrayList<Object>());
      ob.pE(new ArrayList<String>());
      printf("-------------------------------%n");
      ob.pF(new IKD(11));
      ob.pF(new IKE(12));
      ob.pF("Hallo pF!");
      printf("-------------------------------%n");
      ob.pG(new IKB());
      printf("-------------------------------%n");
      ob.pH(new IKC(21));
      ob.pH(new IKD(22));
      ob.pH(new IKE(23));
//    ob.pH(new IKF(24));                          // compile time error
      ob.pH(new String("Hallo pH!"));
      printf("-------------------------------%n");
//    ob.pI(new IKC(31));                          // compile time error
      ob.pI(new IKD(32));
      ob.pI(new IKE(33));
      ob.pI(new IKF(34));
      ob.pI(new String("Hallo pI!"));
      printf("-------------------------------%n");
      ob.pJ(new ArrayList<String>(), new IKG());
      ob.pJ(new ArrayList<String>(), new IKH());
      printf("-------------------------------%n");
      printf("GenParams01: Das war's erstmal!%n");
   } // main
   // ---------------------------------------------------------------------
   // Mehrere Methoden mit kurzen Namen:
   static void pln(Object ob) {System.out.println(ob);}
   static void printf(String f, Object... v) {System.out.printf(f, v);}
   // ---------------------------------------------------------------------
} // class GenParams01
/* ------------------------------------------------------------------------
Ausgabe:

GenParams01: Jetzt geht es los!
-------------------------------
pA: Hallo pA!
pA: 123.4
pA: 234.5
pA: []
-------------------------------
pB: 2.5
pB: 123
-------------------------------
pC: IKA 0
-------------------------------
pD: []
pD: []
-------------------------------
pE: []
pE: []
-------------------------------
pF: IKD 11
pF: IKE 12
pF: Hallo pF!
-------------------------------
pG: IKB
-------------------------------
pH: IKC 21
pH: IKD 22
pH: IKE 23
pH: Hallo pH!
-------------------------------
pI: IKD 32
pI: IKE 33
pI: IKF 34
pI: Hallo pI!
-------------------------------
pI: []
pI: []
-------------------------------
GenParams01: Das war's erstmal!
------------------------------------------------------------------------ */
