// Datei MeineSammlungTst.java
/* ------------------------------------------------------------------------
Dieses Programm dient zum Testen der generischen Klasse MeineSammlung<K>.
------------------------------------------------------------------------ */
import java.util.Collection;
import java.util.Iterator;
import java.util.Arrays;

class MeineSammlungTst {
   // ---------------------------------------------------------------------
   static public void main(String[] sonja) {
      printf("MeineSammlungTst: Jetzt geht es los!%n");
      printf("------------------------------------%n");

      // Reihungen mit den Daten, die gesammelt werden sollen:
      String [] sr = {"AB", "CDEF", "GHI"};
      Integer[] ir = {-17, +25, -34, +86};

      Collection<String>  sc = Arrays.asList(sr);
      Collection<Integer> ic = Arrays.asList(ir);

      // Zwei Sammlungsobjekte:
      MeineSammlung<String>  ms10 = new MeineSammlung<String> (sr);
      MeineSammlung<Integer> ms20 = new MeineSammlung<Integer>(ic);

      printf("A ms10.hashCode(): %d%n", ms10.hashCode());
      printf("B ms20.hashCode(): %d%n", ms20.hashCode());
      printf("------------------------------------%n");

      // Das Sammlungsobjekt ms10 wir ein bisschen getestet:
      printf("C ms10: %s%n", ms10.toString());
      printf("D ms10.contains(\"CDEF\"): %b%n", ms10.contains("CDEF"));
      printf("E ms10.contains(\"XYXY\"): %b%n", ms10.contains("XYXY"));
      printf("F ms10: %s%n", ms10.toString());
      printf("------------------------------------%n");
      // Mit Hilfe eines Iterator-Objekts werden alle Komponenten
      // der Sammlung ms10 ausgegeben:
      for (Iterator<String> is=ms10.iterator(); is.hasNext(); ) {
         printf("G Naechste Komponente: %s%n", is.next());
      }
      printf("------------------------------------%n");

      // Die beiden toArray-Methoden ausprobieren:
      Object[] sr2 = ms10.toArray();
      printf("H sr2: %s%n", Arrays.toString(sr2));
      String[] sr3 = {"X1", "X2", "X3", "X4", "X5"};
      String[] sr4 = ms10.toArray(sr3);
      printf("I sr3: %s%n", Arrays.toString(sr3));
      printf("------------------------------------%n");

      // Das Sammlungsobjekt ms20 wir ein bisschen getestet:
      printf("J ms20: %s%n", ms20.toString());
      printf("K ms20.contains(-34): %b%n", ms20.contains(-34));
      printf("L ms20.contains(+99): %b%n", ms20.contains(+99));
      printf("M ms20: %s%n", ms20.toString());
      printf("------------------------------------%n");
      // Mit Hilfe einer sicheren for-Schleife werden all Komponenten
      // der Sammlung ms20 ausgegeben:
      for (Integer iob : ms20) {
         printf("N Naechste Komponente: %+d%n", iob);
      }
      printf("------------------------------------%n");

      // Die beiden toArray-Methoden ausprobieren:
      Object[] ir2 = ms20.toArray();
      printf("O ir2: %s%n", Arrays.toString(ir2));
      Integer[] ir3 = {11, 22, 33, 44, 55, 66, 77};
      Integer[] ir4 = ms20.toArray(ir3);
      printf("P ir3: %s%n", Arrays.toString(ir3));
      printf("------------------------------------%n");
      // Ein paar MeineSammlung-Objekte erzeugen, zum Testen von equals:
      MeineSammlung<String>  ms11 = new MeineSammlung<String> (ms10);
      MeineSammlung<String>  ms12 = new MeineSammlung<String> (sr);
      MeineSammlung<Integer> ms21 = new MeineSammlung<Integer>(ms20);
      MeineSammlung<Integer> ms22 = new MeineSammlung<Integer>(ir);

      printf("Q ms10: %s%n", ms10.toString());
      printf("Q ms11: %s%n", ms11.toString());
      printf("Q ms12: %s%n", ms12.toString());

      printf("Q ms10.equals(ms10): %b%n", ms10.equals(ms10));
      printf("R ms10.equals(ms11): %b%n", ms10.equals(ms11));
      printf("S ms10.equals(ms12): %b%n", ms10.equals(ms12));

      printf("Q ms20: %s%n", ms20.toString());
      printf("Q ms21: %s%n", ms21.toString());
      printf("Q ms22: %s%n", ms22.toString());

      printf("T ms20.equals(ms20): %b%n", ms20.equals(ms20));
      printf("U ms20.equals(ms21): %b%n", ms20.equals(ms21));
      printf("V ms20.equals(ms22): %b%n", ms20.equals(ms22));
      printf("------------------------------------%n");
      printf("MeineSammlungTst: Das war's erstmal!%n");
   } // main
   // ---------------------------------------------------------------------
   // Eine Methode mit einem kurzen Namen:
   static void printf(String f, Object... v) {System.out.printf(f, v);}
   // ---------------------------------------------------------------------
} // class MeineSammlungTst
/* ------------------------------------------------------------------------
Ausgabe des Programms MeineSammlungTst:

MeineSammlungTst: Jetzt geht es los!
------------------------------------
A ms10.hashCode(): 66070342
B ms20.hashCode(): 440131
------------------------------------
C ms10: [AB, CDEF, GHI]
D ms10.contains("CDEF"): true
E ms10.contains("XYXY"): false
F ms10: [CDEF, AB, GHI]
------------------------------------
G Naechste Komponente: CDEF
G Naechste Komponente: AB
G Naechste Komponente: GHI
------------------------------------
H sr2: [CDEF, AB, GHI]
I sr3: [CDEF, AB, GHI, null, X5]
------------------------------------
J ms20: [-17, 25, -34, 86]
K ms20.contains(-34): true
L ms20.contains(+99): false
M ms20: [-34, 25, -17, 86]
------------------------------------
N Naechste Komponente: -34
N Naechste Komponente: +25
N Naechste Komponente: -17
N Naechste Komponente: +86
------------------------------------
O ir2: [-34, 25, -17, 86]
P ir3: [-34, 25, -17, 86, null, 66, 77]
------------------------------------
Q ms10.equals(ms10): true
R ms10.equals(ms11): true
S ms10.equals(ms12): false
T ms20.equals(ms20): true
U ms20.equals(ms21): true
V ms20.equals(ms22): false
------------------------------------
MeineSammlungTst: Das war's erstmal!
------------------------------------------------------------------------ */
