// Datei CreateDom10.java
/* ------------------------------------------------------------------------
Dieses Programm CreateDom10 leistet Folgendes:
------------------------------------------------------------------------ */
import org  .w3c.dom          .Document;
import org  .w3c.dom          .DocumentType;
import org  .w3c.dom          .Element;
import org  .w3c.dom          .Node;
import org  .w3c.dom          .Text;
import org  .w3c.dom          .Comment;
import org  .w3c.dom          .DOMImplementationList;
import org  .w3c.dom.bootstrap.DOMImplementationRegistry;
import org  .w3c.dom          .DOMImplementation;

class CreateDom10 {
   // ---------------------------------------------------------------------
   static public Document createDom10(String pfad) {
      // Erzeugt ein DOM-Objekt und liefert es als Ergebnis.
      // Der pfad wird nur zur Erzeugung eines Kommentars am Anfang
      // des DOM-Objekts verwendet.

      try {

         DOMImplementation impl = besorgeEineDomImpl();

         // Das Dokument wird mit einem Wurzelelement "name" erzeugt:
         Document     domDoc10 = impl.createDocument(
            null,    // URI des Namensraums     (oder null)
            "name",  // Name des Wurzelelements (ein qualifizierter Name)
            null     // Ein DocumentType-Objekt (oder null)
         );


         // Die Bestandteile des Dokuments werden erzeugt:
         Comment anfKomm   = domDoc10.createComment ("Datei " + pfad);
         Element vorNameE  = domDoc10.createElement ("vor_name");
         Element nachNameE = domDoc10.createElement ("nach_name");
         Text    vorNameT  = domDoc10.createTextNode("Alan");
         Text    nachNameT = domDoc10.createTextNode("Turing");

         // Die Bestandteile werden in das Dokument eingefuegt:
         Node    wurzel    = domDoc10.getDocumentElement();

         domDoc10.insertBefore(anfKomm, wurzel);

         wurzel.appendChild(vorNameE);
         wurzel.appendChild(nachNameE);

         vorNameE .appendChild(vorNameT);
         nachNameE.appendChild(nachNameT);

         return domDoc10;
      } catch (Exception ex) {
         meldeAusnahme(ex, "CreateDom10");
         return null;
      }
   } // CreateDom10
   // ---------------------------------------------------------------------
   static public Document createDom11(String pfad) {
      // Erzeugt ein DOM-Objekt und liefert es als Ergebnis.
      // Der pfad wird nur zur Erzeugung eines Kommentars am Anfang
      // des DOM-Objekts verwendet.
      //
      // Verstehe das Ergebnis dieser Methode nicht. Diese Methode
      // enthaelt mehr Befehle als die vorige, liefert aber offenbar
      // ein ganz aehnliches Ergebnis.

      try {

         DOMImplementation impl = besorgeEineDomImpl();

         DocumentType docTyp  = impl.createDocumentType(
            "svg",
            "-/W3C/DTD SVG 1.0/EN",
            "http://www.w3.org/TR/2001/REC-SVG-20010904/DTD/svg10.dtd"
         );

         // Das Dokument wird mit einem Wurzelelement "name" erzeugt:
         Document     domDoc11 = impl.createDocument(
            "http://www.w3.org/2000/svg",    // URI des Namensraums     (oder null)
            "svg",  // Name des Wurzelelements (ein qualifizierter Name)
            docTyp  // Ein DocumentType-Objekt (oder null)
         );

         domDoc11.setXmlStandalone(true);


         // Die Bestandteile des Dokuments werden erzeugt:
         Comment anfKomm   = domDoc11.createComment ("Datei " + pfad);
         Element vorNameE  = domDoc11.createElement ("vor_name");
         Element nachNameE = domDoc11.createElement ("nach_name");
         Text    vorNameT  = domDoc11.createTextNode("Alan");
         Text    nachNameT = domDoc11.createTextNode("Turing");

         // Die Bestandteile werden in das Dokument eingefuegt:
         Node    wurzel    = domDoc11.getDocumentElement();

         domDoc11.insertBefore(anfKomm, wurzel);

         wurzel.appendChild(vorNameE);
         wurzel.appendChild(nachNameE);

         vorNameE .appendChild(vorNameT);
         nachNameE.appendChild(nachNameT);

         return domDoc11;
      } catch (Exception ex) {
         meldeAusnahme(ex, "CreateDom10");
         return null;
      }
   } // CreateDom11
   // ---------------------------------------------------------------------
   static public DOMImplementation besorgeEineDomImpl() throws Exception {
      // Wir suchen eine DOM-Implementierung mit den drei
      // Eigenschaften "XML 1.0", "+Traversal" und "+Events 2.0"
      // (die beiden Pluszeichen + sind optional):
      String domFeatures = "XML 1.0 +Traversal +Events 2.0";
      DOMImplementationRegistry registry =
         DOMImplementationRegistry.newInstance();
      DOMImplementation         impl     = (DOMImplementation)
         registry.getDOMImplementation(domFeatures);

      // Ohne DOM-Implementierung koennen wir hier nichts machen:
      if (impl==null) throw new Exception(
         "Konnte keine DOM-Implementierung mit den Eigenschaften \"" +
         domFeatures + "\" finden!"
      );

      return impl;
   } // besorgeEineDomImpl
   // ---------------------------------------------------------------------
   static void meldeAusnahme(Exception ex, String inMethode) {
      String ausnahme = ex.getClass().getName();
      printf("In XmlJStd.%s: %s%n", inMethode, ausnahme);
      printf("Message: %s%n", ex.getMessage());
   } // meldeAusnahme
   // ---------------------------------------------------------------------
   static public void gibAus(Document domDoc, String pfad) {
      XmlJStd.schreib(domDoc, pfad);
      printf("--------------------------------------------------------%n");
      printf("Ein DOM-Dokument wurde in die Datei %s%n", pfad);
      printf("geschrieben und sieht dort so aus:%n%n");
      XmlJStd.gibTextDateiAus  (pfad);
   } // gibAus
   // ---------------------------------------------------------------------
   static public void main(String[] sonja) {
      printf("CreateDom10: Jetzt geht es los!%n");

      erzeugeUndGibAus(0);
      erzeugeUndGibAus(1);

      printf("--------------------------------------------------------%n");
      printf("CreateDom10: Das war's erstmal!%n");
   } // main
   // ---------------------------------------------------------------------
   static Document erzeugeUndGibAus(int ind) {
      String[] pfade = {
         "dDom10A.xml",
         "dDom11A.xml",
      };

      printf("--------------------------------------------------------%n");
      if (ind < 0 || pfade.length <= ind) {
         printf("CreateDom10.erzeugeUndGibAus: Falscher Index %d%n", ind);
         return null;
      }

      Document domDoc = null;
      String   pfad    = pfade[ind];

      printf("Ein DOM-Dokument wird erzeugt und in die Datei %s%n", pfad);
      printf("geschrieben. Dort sieht es so aus:%n%n");

      switch (ind) {
         case 0: domDoc = createDom10 (pfad); break;
         case 1: domDoc = createDom11 (pfad); break;
      }
      XmlJStd.schreib(domDoc, pfad);
      XmlJStd.gibTextDateiAus (pfad);

      return domDoc;
   } // erzeugeUndGibAus
   // ---------------------------------------------------------------------
   // Eine Methode mit einem kurzen Namen:
   static void printf(String f, Object... v) {System.out.printf(f, v);}
   // ---------------------------------------------------------------------
} // class CreateDom10
/* ------------------------------------------------------------------------
Die Ausgabe des Programms CreateDom10.java (2007-09-14):

CreateDom10: Jetzt geht es los!
--------------------------------------------------------
Ein DOM-Dokument wird erzeugt und in die Datei dDom10A.xml
geschrieben. Dort sieht es so aus:

<?xml version="1.0" encoding="UTF-8"?>
<!--Datei dDom10A.xml-->
<name>
    <vor_name>Alan</vor_name>
    <nach_name>Turing</nach_name>
</name>
--------------------------------------------------------
Ein DOM-Dokument wird erzeugt und in die Datei dDom11A.xml
geschrieben. Dort sieht es so aus:

<?xml version="1.0" encoding="UTF-8"?>
<!DOCTYPE svg PUBLIC "-/W3C/DTD SVG 1.0/EN"
                     "http://www.w3.org/TR/2001/REC-SVG-20010904/DTD/svg10.dtd">
<!--Datei dDom11A.xml-->
<svg xmlns="http://www.w3.org/2000/svg">
    <vor_name>Alan</vor_name>
    <nach_name>Turing</nach_name>
</svg>
--------------------------------------------------------
CreateDom10: Das war's erstmal!
------------------------------------------------------------------------ */
