// Datei CreateJdom10.java
/* ------------------------------------------------------------------------
Dieses Programm CreateJdom10 leistet Folgendes:
------------------------------------------------------------------------ */
import org.jdom.Document;
import org.jdom.DocType;
import org.jdom.Element;
import org.jdom.Content;
import org.jdom.Comment;
import org.jdom.Text;
import org.jdom.CDATA;

class CreateJdom10 {
   // ---------------------------------------------------------------------
   static Document createJdom10(String pfad) {
      // Erzeugt ein JDOM-Objekt und liefert es als Ergebnis.
      // Der pfad wird nur zur Erzeugung eines Kommentars am Anfang
      // des JDOM-Objekts verwendet.

      try {
         // Die Bestandteile des Dokuments erzeugen:
         Element wurzelElem   = new Element("name");
         Element vorNameElem  = new Element("vor_name");
         Element nachNameElem = new Element("nach_name");

         // Elemente mit Inhalt fuellen:
         nachNameElem.setText("Turing");
         vorNameElem .setText("Alan");

         // Elemente zusammensetzen:
         wurzelElem.addContent(vorNameElem);
         wurzelElem.addContent(nachNameElem);

         // Das Dokument selbst erzeugen:
         Document jdomDoc10 = new Document(wurzelElem);

         return jdomDoc10;
      } catch (Exception ex) {
         meldeAusnahme(ex, "CreateJdom10");
         return null;
      }
   } // createJdom10
   // ---------------------------------------------------------------------
   static Document createJdom11(String pfad) {
      // Erzeugt ein JDOM-Objekt und liefert es als Ergebnis.
      // Der pfad wird nur zur Erzeugung eines Kommentars am Anfang
      // des JDOM-Objekts verwendet.
      //
      // Fuegt zu einem fertigen Objekt eine Dokumenten-Typ Deklaration
      // (DTD), einen Anfangskommentar und ein (ungueltiges) zweites
      // vor_name-Element hinzu.
      //
      // ACHTUNG:
      // Das Hinzufuegen einer DTD bewirkt NICHT automatisch, dass das
      // Dokument auf Gueltigkeit geprueft wird! Man kann es dann aber
      // durch extra-Befehle auf Gueltigkeit pruefen lassen.

      try {
         // Ein fertiges Dokument besorgen:
         Document jdomDoc11 = createJdom10(pfad);

         // Bestandteile der Erweiterung erzeugen:
         DocType  docTyp   = new DocType("name", "nameA.dtd");
         Comment  anfKomm  = new Comment(pfad);

         // Neue Bestandteile einbauen:
         jdomDoc11.setDocType(docTyp);
         jdomDoc11.addContent(0, anfKomm); // Ganz am Anfang einfuegen

         // Einen zweites vor_name-Element einfuegen (ungueltig!):
         Element wurzel   = jdomDoc11.getRootElement();
         Element vorName2 = new Element("vor_name");
         vorName2.setText("Mathison");
         wurzel.addContent(1, vorName2);

         return jdomDoc11;
      } catch (Exception ex) {
         meldeAusnahme(ex, "CreateJdom10");
         return null;
      }
   } // createJdom11
   // ---------------------------------------------------------------------
   static Document createJdom12(String pfad) {
      // Erzeugt ein JDOM-Objekt und liefert es als Ergebnis.
      // Der pfad wird nur zur Erzeugung eines Kommentars am Anfang
      // des JDOM-Objekts verwendet.
      //
      // Fuegt zu einem fertigen Objekt eine Dokumenten-Typ Deklaration
      // (DTD), einen Anfangskommentar und ein (ungueltiges) zweites
      // vor_name-Element hinzu und VALIDIERT das fertige Dokument
      // dann gegen seine DTD.

      try {
         // Ein fertiges Dokument besorgen:
         Document jdomDoc12 = createJdom10(pfad);

         // Bestandteile der Erweiterung erzeugen:
         DocType  docTyp   = new DocType("name", "nameA.dtd");
         Comment  anfKomm  = new Comment(pfad);

         // Neue Bestandteile einbauen:
         jdomDoc12.setDocType(docTyp);
         jdomDoc12.addContent(0, anfKomm); // Ganz am Anfang einfuegen

         // Einen zweites vor_name-Element einfuegen (ungueltig):
         Element wurzel   = jdomDoc12.getRootElement();
         Element vorName2 = new Element("vor_name");
         vorName2.setText("Mathison");
         wurzel.addContent(1, vorName2);

         // Das jdomDoc12 validieren und Ergebnis ausgeben:
         printf("jdomDoc12 wird auf Gueltigkeit geprueft:%n");
         boolean erg = XmlJdom.istGueltigVD(jdomDoc12);
         printf("Das Ergebnis der Pruefung: %b%n%n", erg);

         return jdomDoc12;
      } catch (Exception ex) {
         meldeAusnahme(ex, "CreateJdom10");
         return null;
      }
   } // createJdom12
   // ---------------------------------------------------------------------
   static public void meldeAusnahme(Exception ex, String inMethode) {
      String ausnahme = ex.getClass().getName();
      printf("In XmlJStd.%s: %s%n", inMethode, ausnahme);
      printf("Message: %s%n", ex.getMessage());
   } // meldeAusnahme
   // ---------------------------------------------------------------------
   static public void main(String[] sonja) {
      printf("CreateJdom10: Jetzt geht es los!%n");

      erzeugeUndGibAus(0);
      erzeugeUndGibAus(1);
      erzeugeUndGibAus(2);

      printf("--------------------------------------------------------%n");
      printf("CreateJdom10: Das war's erstmal!%n");
   } // main
   // ---------------------------------------------------------------------
   static Document erzeugeUndGibAus(int ind) {
      String[] pfade = {
         "dJdom10A.xml",
         "dJdom11A.xml",
         "dJdom12A.xml",
      };

      printf("--------------------------------------------------------%n");
      if (ind < 0 || pfade.length <= ind) {
         printf("CreateJdom10.erzeugeUndGibAus: Falscher Index %d%n", ind);
         return null;
      }

      Document jdomDoc = null;
      String   pfad    = pfade[ind];

      printf("Ein JDOM-Dokument wird erzeugt und in die Datei %s%n", pfad);
      printf("geschrieben. Dort sieht es so aus:%n%n");

      switch (ind) {
         case 0: jdomDoc = createJdom10 (pfad); break;
         case 1: jdomDoc = createJdom11 (pfad); break;
         case 2: jdomDoc = createJdom12 (pfad); break;
      }
      XmlJdom.schreib(jdomDoc, pfad);
      XmlJdom.gibTextDateiAus (pfad);

      return jdomDoc;
   } // erzeugeUndGibAus
   // ---------------------------------------------------------------------
   // Eine Methode mit einem kurzen Namen:
   static void printf(String f, Object... v) {System.out.printf(f, v);}
   // ---------------------------------------------------------------------
} // class CreateJdom10
/* ------------------------------------------------------------------------
Die Ausgabe des Programms CreateJdom10.java (2007-09-14):

CreateJdom10: Jetzt geht es los!
--------------------------------------------------------
Ein JDOM-Dokument wird erzeugt und in die Datei dJdom10A.xml
geschrieben. Dort sieht es so aus:

<?xml version="1.0" encoding="UTF-8"?>
<name>
  <vor_name>Alan</vor_name>
  <nach_name>Turing</nach_name>
</name>

--------------------------------------------------------
Ein JDOM-Dokument wird erzeugt und in die Datei dJdom11A.xml
geschrieben. Dort sieht es so aus:

<?xml version="1.0" encoding="UTF-8"?>
<!--dJdom11A.xml-->
<!DOCTYPE name SYSTEM "nameA.dtd">

<name>
  <vor_name>Alan</vor_name>
  <vor_name>Mathison</vor_name>
  <nach_name>Turing</nach_name>
</name>

--------------------------------------------------------
Ein JDOM-Dokument wird erzeugt und in die Datei dJdom12A.xml
geschrieben. Dort sieht es so aus:

jdomDoc12 wird auf Gueltigkeit geprueft:
In XmlJdom.istGueltigVD: org.jdom.input.JDOMParseException
Message: Error on line 3: The content of element type "name" must match "(vor_name,nach_name)".
Das Ergebnis der Pruefung: false

<?xml version="1.0" encoding="UTF-8"?>
<!--dJdom12A.xml-->
<!DOCTYPE name SYSTEM "nameA.dtd">

<name>
  <vor_name>Alan</vor_name>
  <vor_name>Mathison</vor_name>
  <nach_name>Turing</nach_name>
</name>

--------------------------------------------------------
CreateJdom10: Das war's erstmal!
------------------------------------------------------------------------ */
