// Datei ErsetzeKommentar.java
/* ------------------------------------------------------------------------
Dieses Programm ErsetzeKommentar erwartet 2 Pfadnamen als Kommendozeilen-
parameter. Ein Aufruf sieht etwa so aus:

> java ErsetzeKommentar Hallo.java tmp.txt

Der erste Pfadname sollte eine Java-Quell-Datei (.java-Datei) bezeichnen,
die mit einem Schaegstrich-Stern-Kommentar endet. Der zweite Pfadname
sollte eine Textdatei bezeichnen.

Der Kommentar am Ende der Java-Quell-Datei wird durch den Inhalt der
Textdatei ersetzt.

Ein typische Anwendung dieses Programms sieht etwa so aus:

> java Prog27 > tmp.txt
> java ErsetzeKommentar Prog27.java tmp.txt

Danach enthaelt die Quelldatei Prog27.java die Ausgabe des Programms Prog27.
------------------------------------------------------------------------ */
import java.io  .File;
import java.io  .RandomAccessFile;
import java.io  .IOException;
//import java.util.Date;
import java.sql .Date;

class ErsetzeKommentar {
   // ---------------------------------------------------------------------
   static public void main(String[] sonja) throws IOException {

      RandomAccessFile rafJ = null;
      RandomAccessFile rafK = null;
      byte[]           byrJ;
      byte[]           byrK;

      // Kommandozeilen-Parameter pruefen:
      if (sonja.length!=2) {
         printf("Usage:%n");
         printf("java ErsetzeKommentar JavaQuellDatei TextDatei%n");
         return;
      }

      if (!sonja[0].endsWith(".java")) {
         printf("ErsetzeKommentar: Die Datei %s scheint%n", sonja[0]);
         printf("keine Java-Quelldatei (.java-Datei) zu sein!%n");
         return;
      }

      try {
         // "J" wie "Java-Datei" und "K" wie "neuer Kommentar":
         String pfadJ = sonja[0];
         String pfadK = sonja[1];

         printf("ErsetzeKommentar bearbeitet die Dateien %s und %s%n",
            pfadJ, pfadK);

         File   fileJ = new File(pfadJ);
         File   fileK = new File(pfadK);

         rafJ = new RandomAccessFile(pfadJ, "rw");
         rafK = new RandomAccessFile(pfadK, "r");

         byrJ = liesRAF(rafJ, pfadJ);
         byrK = liesRAF(rafK, pfadK);

         int von = beginnDesLetztenKommentars(byrJ);
         if (von==-1) meldeFehlerKeinKommentar(pfadJ);

         rafJ.setLength(von);
         rafJ.seek(von);

         String minZ = "------------------------------------" +
                       "------------------------------------";
         String kommentarAnfang = "/* " +  minZ;
         String kommentarEnde   = minZ  + " */";

         long jetzt = System.currentTimeMillis();
         StringBuilder sb = new StringBuilder();
         sb.append("Die Ausgabe des Programms ");
         sb.append(pfadJ);
         sb.append(" (");
         sb.append(new Date(jetzt).toString());
         sb.append("):");

         println(kommentarAnfang, rafJ);
         println(sb.toString(),   rafJ);
         println("",              rafJ);
         rafJ.write(byrK);
         println(kommentarEnde,   rafJ);
      } catch (Exception ex) {
         printf("In ErsetzeKommentar: %s%n", ex.getClass().getName());
         printf("Meldung: %s%n", ex.getMessage());
      } finally {
         rafJ.close();
         rafK.close();
      }
   } // main
   // ---------------------------------------------------------------------
   static int beginnDesLetztenKommentars(byte[] byr) {
      // byr sollte den Text einer Java-Quelldatei enthalten, die
      // mit einem /* ... */ - Kommentar endet. Liefert den Index
      // des Schraegstrichs /, mit dem dieser Kommentar beginnt,
      // und -1, falls ein solcher Kommentar nicht gefunden wird.

      int erg = byr.length; // Ein Index fuer byr

      // Transparente Zeichen am Ende von byr ignorieren:
      while (erg>0 && Character.isWhitespace(byr[--erg]));

      // Jetzt muesste ab byr[erg-1] das Kommentar-Ende "*/" stehen:
      if (erg<4 || byr[erg-1]!='*' || byr[erg]!='/') {
         printf("erg: %d,  byr[erg, erg+1]: %c%c%n", erg,  byr[erg],  byr[erg+1]);
         return -1;  // Kein Kommentar-Ende   "*/" gefunden
      }
      erg -= 2;

      // Den Kommentar-Anfang "/*" suchen:
      while (erg>=0 && (byr[erg-1]!='/' || byr[erg]!='*')) erg--;
      if (erg<0) {
         printf("erg: %d,  byr[erg, erg+1]: %c%c%n", erg,  byr[erg],  byr[erg+1]);
         return -1;  // Kein Kommentar-Anfang "/*" gefunden
      }

      return erg-1;
   } // beginnDesLetztenKommentars
   // ---------------------------------------------------------------------
   static byte[] liesRAF(RandomAccessFile raf, String pfad)
      throws IOException
  {
      // Liest alle Bytes der Datei raf in eine Reihung byr und liefert
      // die als Ergebnis. Der zweite Param sollte den Pfad der Datei
      // raf enthalten. Er wird nur gebraucht, wenn eine Fehlermeldung
      // ausgegeben wird.

      byte[] byr = new byte[(int) raf.length()];
      int    len = raf.read(byr);

      if (len != raf.length()) {
         String pn = "ErsetzeKommentar";
         printf("%s: Fehler beim Lesen der Datei %s:%n", pn, pfad);
         printf("Dateilaenge ist     %d%n",              raf.length());
         printf("Gelesen wurden aber %d Bytes!%n",       len);
         System.exit(1);
      }
      return byr;
   } // liesRAF
   // ---------------------------------------------------------------------
   static void meldeFehlerKeinKommentar(String pfad) {
      String pn = "ErsetzeKommentar";
      printf("%s: Die Datei %s scheint nicht mit %n", pn, pfad);
      printf("einem /* ... */ - Kommentar zu enden%n");
      System.exit(2);
   } // meldeFehlerKeinKommentar
   // ---------------------------------------------------------------------
   static final String LS = System.getProperty("line.separator");
   static void println(String s, RandomAccessFile raf) throws IOException {
      for (int i=0; i< s.length(); i++) raf.write( s.charAt(i));
      for (int i=0; i<LS.length(); i++) raf.write(LS.charAt(i));
   } // println
   // ---------------------------------------------------------------------
   // Eine Methode mit einem kurzen Namen:
   static void printf(String f, Object... v) {System.out.printf(f, v);}
   // ---------------------------------------------------------------------
} // class ErsetzeKommentar
