// Datei XmlJdom.java
/* ------------------------------------------------------------------------
Dieser Modul enthaelt Methoden zum Bearbeiten von XML-Dateien und
benutzt dazu die Standardbibliothek von Java 6 und zusaetzlich JDOM.

Die wichtigen Methoden:
lies             Liest eine XML-Datei und erzeugt daraus ein
                 entsprechendes JDOM-Document-Objekt.
istGueltigVD     Prueft, ob ein JDOM-Document-Objekt gueltig ist
                 gegen seine DTD.
istGueltigVS     Prueft, ob ein JDOM-Document-Objekt gueltig ist
                 gegen eine XML-Schema-Datei.
schreib          Schreibt ein JDOM-Document-Objekt im XML-Format
                 in eine Datei

Weniger wichtige Methoden:
gibTextDateiAus  Gibt eine beliebige Textdatei zur Standardausgabe aus
gibLesbarAus     Gibt ein JDOM-Document-Objekt in einer selbstgestrickten
                 Form ("als Einrueck-Baum") zur Standardausgabe aus.
istGueltigVD2    Funktioniert (noch?) nicht.
istGueltigVD3    Funktioniert (noch?) nicht.
istGueltigVS2    Funktioniert (noch?) nicht.
istGueltigVS3    Funktioniert (noch?) nicht.

Achtung: Dieser Modul benutzt den Modul XmlJStd.java!
------------------------------------------------------------------------ */
import java .io                    .File;
import java .io                    .FileReader;
import java .io                    .FileOutputStream;
import java .io                    .StringWriter;
import java .io                    .StringReader;
import java .io                    .PipedWriter;
import java .io                    .PipedReader;
import java .io                    .PipedOutputStream;
import java .io                    .PipedInputStream;
import java .util                  .List;

import javax.xml                   .XMLConstants;
import javax.xml .validation       .SchemaFactory;
import javax.xml .validation       .Schema;
import javax.xml .validation       .Validator;
import javax.xml .transform .stream.StreamSource;

import org  .jdom                  .Document;
import org  .jdom                  .Element;
import org  .jdom                  .Content;
import org  .jdom                  .Comment;
import org  .jdom                  .Text;
import org  .jdom                  .CDATA;
import org  .jdom                  .EntityRef;
import org  .jdom                  .ProcessingInstruction;
import org  .jdom.input            .SAXBuilder;
import org  .jdom.output           .XMLOutputter;
import org  .jdom.output           .DOMOutputter;
import org  .jdom.output           .Format;

public class XmlJdom {
   // ---------------------------------------------------------------------
   public static Document lies(String pfadXml) {
      // Liest die XML-Datei pfadXml ein, macht daraus ein JDOM-Document-
      // Objekt und liefert dieses als Ergebnis.
      // Falls eine Ausnahme auftritt, wird eine Fehlermeldung ausgegeben
      // und null als Ergebnis geliefert.
      // Beim Einlesen wird die XML-Datei NICHT auf Gueltigkeit geprueft,
      // nur auf Wohlgeformtheit.

      try {
         SAXBuilder builder = new SAXBuilder(false); // Validiert nicht
         File       file    = new File(pfadXml);
         Document   jdomDoc = builder.build(file);
         return jdomDoc;
      } catch (Exception ex) {
         meldeAusnahme(ex, "lies");
         return null;
      }
   } // lies
   // ---------------------------------------------------------------------
   public static boolean istGueltigVD(Document jdomDoc) {
      // Validiert jdomDoc gegen seine Dokumenten-Typ-Definition (DTD).
      // Liefert true genau dann wenn jdomDoc gueltig ist.
      // Falls eine Ausnahme auftritt, wird eine Fehlermeldung ausgegeben
      // und false als Ergebnis geliefert. "VD" wie "validiert gegen DTD".
      //
      // Dazu wird das jdomDoc in ein StringWriter-Objekt "rausgeschrieben"
      // und dann mit einem StringReader-Objekt (und einem validierenden
      // Parser) wieder "reingelesen".

      try {
         XMLOutputter rausPutter   = new XMLOutputter();
         StringWriter stringWriter = new StringWriter();
         rausPutter.output(jdomDoc, stringWriter);
         String       writerInhalt = stringWriter.toString();
         StringReader stringReader = new StringReader(writerInhalt);

         SAXBuilder   reinBuilder  = new SAXBuilder(true); // Validiert
         reinBuilder.build (stringReader);

         return true;                    // jdomDoc ist gueltig
      } catch (Exception ex) {
         meldeAusnahme(ex, "istGueltigVD");
         return false;                   // jdomDoc ist nicht gueltig
      }
   } // istGueltigVD
   // ---------------------------------------------------------------------
   public static boolean istGueltigVD2(Document jdomDoc) {
      // Validiert jdomDoc gegen seine Dokumenten-Typ-Definition (DTD).
      // Liefert true genau dann wenn jdomDoc gueltig ist.
      // Falls eine Ausnahme auftritt, wird eine Fehlermeldung ausgegeben
      // und false als Ergebnis geliefert. "VD" wie "validiert gegen DTD".
      //
      // Dazu wird das jdomDoc in ein PipedWriter-Objekt "rausgeschrieben"
      // und dann mit einem PipedReader-Objekt (und einem validierenden
      // Parser) wieder "reingelesen".
      // FUNKTIONIERT NICHT: Bleibt beim Lesen aus dem PipedReader-Objekt
      // "haengen" und macht nichts mehr. Was habe ich falsch gemacht?

      try {
         XMLOutputter rausPutter  = new XMLOutputter();
         SAXBuilder   reinBuilder = new SAXBuilder(true); // Validiert

         PipedWriter  pipedWriter = new PipedWriter();
         PipedReader  pipedReader = new PipedReader(pipedWriter);
         printf("### A pipedReader.ready(): %b%n", pipedReader.ready());

         // Das jdomDoc wird "rausgeschrieben"
         rausPutter.output(jdomDoc, pipedWriter);
//       pipedWriter.flush();
         printf("### B pipedReader.ready(): %b%n", pipedReader.ready());
         // und wieder "reingelesen" und dabei validiert:
         reinBuilder.build (pipedReader);

         return true;                    // jdomDoc ist gueltig
      } catch (Exception ex) {
         meldeAusnahme(ex, "istGueltigVD2");
         return false;                   // jdomDoc ist nicht gueltig
      }
   } // istGueltigVD2
   // ---------------------------------------------------------------------
   public static boolean istGueltigVD3(Document jdomDoc) {
      // Validiert jdomDoc gegen seine Dokumenten-Typ-Definition (DTD).
      // Liefert true genau dann wenn jdomDoc gueltig ist.
      // Falls eine Ausnahme auftritt, wird eine Fehlermeldung ausgegeben
      // und false als Ergebnis geliefert. "VD" wie "validiert gegen DTD".
      //
      // Dazu wird das jdomDoc in ein PipedOutputStream-Objekt "rausgeschrieben"
      // und dann mit einem PipedInputStream-Objekt (und einem validierenden
      // Parser) wieder "reingelesen".
      // FUNKTIONIERT NICHT: Bleibt beim Lesen aus dem PipedOutputStream-Objekt
      // "haengen" und macht nichts mehr. Was habe ich falsch gemacht?

      try {
         XMLOutputter      rausPutter  = new XMLOutputter();
         SAXBuilder        reinBuilder = new SAXBuilder(true); // Validiert

         PipedOutputStream pipedOS     = new PipedOutputStream();
         PipedInputStream  pipedIS     = new PipedInputStream(pipedOS);

         // Das jdomDoc wird "rausgeschrieben"
         rausPutter.output(jdomDoc, pipedOS);
//       pipedOS.flush();
         printf("### pipedIS.available(): %d%n", pipedIS.available());
         // und wieder "reingelesen" und dabei validiert:
         reinBuilder.build (pipedIS);

         return true;                    // jdomDoc ist gueltig
      } catch (Exception ex) {
         meldeAusnahme(ex, "istGueltigVD3");
         return false;                   // jdomDoc ist nicht gueltig
      }
   } // istGueltigVD3
   // ---------------------------------------------------------------------
   public static boolean istGueltigVS(Document jdomDoc, String pfadXsd) {
      // Validiert jdomDoc gegen das XML-Schema in der Datei pfadXsd.
      // Liefert true genau dann wenn jdomDoc gueltig ist.
      // Falls eine Ausnahme auftritt, wird eine Fehlermeldung ausgegeben
      // und false als Ergebnis geliefert. "VS" wie "validiert gegen Schema".
      //
      // Dazu wird das jdomDoc in ein StringWriter-Objekt "rausgeschrieben"
      // und dann mit einem StringReader-Objekt (und einem validierenden
      // Parser) wieder "reingelesen".

      try {
         XMLOutputter rausPutter   = new XMLOutputter();
         StringWriter stringWriter = new StringWriter();
         rausPutter.output(jdomDoc, stringWriter);
         String       writerInhalt = stringWriter.toString();
         StringReader stringReader = new StringReader(writerInhalt);

         // Ein SchemaFactory-Objekt erzeugen:
         SchemaFactory schemaFactory =
         SchemaFactory.newInstance(XMLConstants.W3C_XML_SCHEMA_NS_URI);

         // Ein Schema-Objekt und ein Validator-Objekt erzeugen:
         File      file      = new File(pfadXsd);
         Schema    schema    = schemaFactory.newSchema(file);
         Validator validator = schema.newValidator();

         // Die Methode validate erwartet ein StreamSource-Objekt:
         StreamSource streamSource = new StreamSource(stringReader);

         // Das XML-Dokument in der streamSource wird auf Gueltigkeit
         // geprueft. Wird dabei ein Fehler festgestellt, so wird eine
         // Ausnahme (des Typs org.xml.sax.SAXParseException) geworfen:
         validator.validate(streamSource);
//       printf("Das zu validierende JDOM-Objekt ist gueltig!%n");
         return true;

      } catch (Exception ex) {
         meldeAusnahme(ex, "istGueltigVS");
         return false;
      }
   } // istGueltigVS
   // ---------------------------------------------------------------------
   public static boolean istGueltigVS2(Document jdomDoc, String pfadXsd) {
      // Validiert jdomDoc gegen das XML-Schema in der Datei pfadXsd.
      // Liefert true genau dann wenn jdomDoc gueltig ist.
      // Falls eine Ausnahme auftritt, wird eine Fehlermeldung ausgegeben
      // und false als Ergebnis geliefert. "VS" wie "validiert gegen Schema".
      //
      // Dazu wird das jdomDoc in ein PipedWriter-Objekt "rausgeschrieben"
      // und dann mit einem PipedReader-Objekt (und einem validierenden
      // Parser) wieder "reingelesen".
      // FUNKTIONIERT NICHT: Bleibt beim Lesen aus dem PipedReader-Objekt
      // "haengen" und macht nichts mehr. Was habe ich falsch gemacht?

      try {
         PipedWriter  pipedWriter  = new PipedWriter();
         PipedReader  pipedReader  = new PipedReader(pipedWriter);

         XMLOutputter rausPutter   = new XMLOutputter();
         StreamSource reinSource   = new StreamSource(pipedReader, pfadXsd);

         rausPutter.output(jdomDoc, pipedWriter);
         printf("jdomDoc wurde in den pipedWriter geschrieben!%n");

         // Ein SchemaFactory-Objekt erzeugen (mit der Klassenmethode
         // (static method) newInstance:
         SchemaFactory schemaFactory =
         SchemaFactory.newInstance(XMLConstants.W3C_XML_SCHEMA_NS_URI);

         // Ein Schema-Objekt und ein Validator-Objekt erzeugen:
         File      file      = new File(pfadXsd);
         Schema    schema    = schemaFactory.newSchema(file);
         Validator validator = schema.newValidator();

         // Die folgenden Test-Befehle zeigen, das das Reader-Objekt
         // pipedReader bereit ist und tatsaechlich die zu validierende
         // XML-Datei enthaelt:
         printf("pfadXsd: %s, file: %s%n", pfadXsd, file.toString());
         printf("Jetzt kann aus pipedReader gelesen und validiert werden!%n");
         printf("pipedReader.ready(): %b%n", pipedReader.ready());
         int z;
//       while ((z=pipedReader.read())!=-1) printf("%c", (char) z);

         // Die XML-Datei in wird auf Gueltigkeit geprueft.
         // Wird dabei ein Fehler festgestellt, so wird eine
         // Ausnahme (des Typs org.xml.sax.SAXParseException) geworfen:
         validator.validate(reinSource);
//       printf("Das zu validierende JDOM-Objekt ist gueltig!%n");
         return true;

      } catch (Exception ex) {
         meldeAusnahme(ex, "istGueltigVS");
         return false;
      }
   } // istGueltigVS2
   // ---------------------------------------------------------------------
   static String jdomF = "dddJdom.xml";
   static String  domF = "ddd_Dom.xml";

   public static boolean istGueltigVS3(Document jdomDoc, String pfadXsd) {
      // Validiert jdomDoc gegen das XML-Schema in der Datei pfadXsd.
      // Liefert true genau dann wenn jdomDoc gueltig ist.
      // Falls eine Ausnahme auftritt, wird eine Fehlermeldung ausgegeben
      // und false als Ergebnis geliefert.

      // Hinweis: JDOM bietet keine Moeglichkeit, Document-Objekte direkt
      // gegen eine Schema-Datei zu validieren. Deshalb wird hier das
      // JDOM-Document-Objekt jdomDoc in ein DOM-Document-Objekt domDoc
      // umgewandelt und domDoc wird validiert.
      // FUNKTIONIERT NICHT:
      // Ausnahme: org.xml.sax.SAXParseException
      // Message: cvc-elt.1: Cannot find the declaration of element 'name'

      try {
         DOMOutputter         dommer = new DOMOutputter();
         org.w3c.dom.Document domDoc = dommer.output(jdomDoc);

         printf("+++++++++++++++++++++++++++++++++++++++++++++++++++%n");
         printf("**0** XmlJdom.istGueltigVS3 ruft XmlJStdVS.istGueltigVS auf:%n");
         printf("+++++++++++++++++++++++++++++++++++++++++++++++++++%n");
         printf("**1** Das originale JDOM-Dokument jdomDoc:%n%n");
                 schreib(jdomDoc, jdomF);
         gibTextDateiAus(jdomF);
         printf("+++++++++++++++++++++++++++++++++++++++++++++++++++%n");
         printf("**2** Das entsprechende DOM-Objekt domDoc:%n%n");
         XmlJStd.schreib( domDoc, domF);
         gibTextDateiAus(domF);
         printf("+++++++++++++++++++++++++++++++++++++++++++++++++++%n");
         printf("**3** Validiert wird gegen die Schema-Datei %s:%n%n", pfadXsd);
         gibTextDateiAus(pfadXsd);
         printf("+++++++++++++++++++++++++++++++++++++++++++++++++++%n");

         return XmlJStd.istGueltigVS(domDoc, pfadXsd);
      } catch (Exception ex) {
         meldeAusnahme(ex, "istGueltigVS3");
         return false;
      }
   } // istGueltigVS3
   // ---------------------------------------------------------------------
   public static void schreib(Document jdomDoc, String pfadXml) {
      // Das Objekt jdomDoc wird als XML-Datei mit dem Pfad
      // pfadXml ausgegeben.

      try {
         FileOutputStream fos        = new FileOutputStream(pfadXml);
         Format           fmt        = Format.getPrettyFormat();
         XMLOutputter     serializer = new XMLOutputter(fmt);
         fmt.setTextMode(Format.TextMode.NORMALIZE); // Optional
         fmt.setIndent("   ");         // Einruecktiefe 3 festlegen

         serializer.output(jdomDoc, fos);
         fos.flush();
         fos.close();
      } catch (Exception ex) {
         meldeAusnahme(ex, "schreib");
      }
   } // schreib
   // ---------------------------------------------------------------------
   static public void gibTextDateiAus(String pfadTxt) {
      // Gibt die Textdatei mit dem Pfad pfadTxt zur Standardausgabe aus.
      try {
         FileReader fr = new FileReader(pfadTxt);
         int zeichen;
         while ((zeichen = fr.read()) != -1) printf("%c", zeichen);
         fr.close();
      } catch (Exception ex) {
         meldeAusnahme(ex, "gibTextDateiAus");
      }
   } // gibTextDateiAus
   // ---------------------------------------------------------------------
   static public void gibLesbarAus(Document jdomDoc) {
      // Gibt das jdomDoc in (hoffentlich) lesbarer Form
      // als "Einrueck-Baum" zur Standardausgabe aus.
      Element wurz = jdomDoc.getRootElement();
      gibLesbarAusR(wurz, ""); // Einrueckung anfangs um 0 Zeichen: ""
   } // gibLesbarAus
   // ---------------------------------------------------------------------
   static public void gibLesbarAusR(Content hier, String einrueck) {
      // Gibt den Baum mit dem Wurzeknoten hier (hoffentlich) lesbar
      // zur Standardausgabe aus (und ruft sich dazu rekursiv auf).
      // Die Baumstruktur wird durch Einrueckung dargestellt. Hinweis:
      // ATTRIBUTE (von Element-Knoten) erscheinen NICHT in diesem Baum.

      if (hier instanceof Element) {
         Element elem = (Element) hier;
         printf("%sElement: ",   einrueck);
         printf("<%s>%n", elem.getName());

         List nachfolger = elem.getContent();
         for (Object nach : nachfolger) {
            gibLesbarAusR((Content) nach, einrueck+"   "); // Rekursion
         }
      } else if (hier instanceof CDATA) {
         // Da CDATA ein Untertyp von Text ist, muss CDATA *vor*
         // Text abgefragt werden!
         CDATA  cdat = (CDATA) hier;
         String wert = cdat.getTextNormalize();
         if (wert.length()>0) {
            printf("%sCDATASection: \"%s\"%n",   einrueck, wert);
         }
      } else if (hier instanceof Text) {
         Text   text = (Text) hier;
         String wert = text.getTextNormalize();
         if (wert.length()>0) {
            printf("%sText: \"%s\"%n",   einrueck, wert);
         }
      } else if (hier instanceof Comment) {
         Comment com = (Comment) hier;
         printf("%sComment: \"%s\"%n", einrueck, com.getText());
      } else if (hier instanceof EntityRef) {
         EntityRef eref = (EntityRef) hier;
         printf("%sEntityRef: \"%s\"%n", einrueck, eref.getName());
      } else if (hier instanceof ProcessingInstruction) {
         ProcessingInstruction proci = (ProcessingInstruction) hier;
         printf("%sProcessingInstruction:%n", einrueck);
         printf("%s   Fuer Programm: %s%n", einrueck, proci.getTarget());
         printf("%s   Instruktion:   %s%n", einrueck, proci.getData());
      } else {
         printf("%sUnbekannter Typ%n", einrueck);
      }
   } // gibLesbarAusR
   // ---------------------------------------------------------------------
   static public void meldeAusnahme(Exception ex, String inMethode) {
      String ausnahme = ex.getClass().getName();
      printf("In XmlJdom.%s: %s%n", inMethode, ausnahme);
      printf("Message: %s%n", ex.getMessage());
   } // meldeAusnahme
   // ---------------------------------------------------------------------
   public static void main (String[] _)  {
      printf("XmlJdom: Jetzt geht es los!%n");
      printf("********************************************************%n");

      String origXml = "daten01B.xml";
      String kopiXml = "dJdom01B.xml";
      String pfadXsd = "nameA.xsd";

      printf("Die Datei %s hat folgenden Inhalt:%n", origXml);
      printf("--------------------------------------------------------%n");
      gibTextDateiAus(origXml);
      printf("--------------------------------------------------------%n");
      printf("Die Datei %s wird in ein JDOM-Objekt eingelesen!%n", origXml);
      Document jdomDoc = lies(origXml);
      printf("Das JDOM-Objekt wird in die Datei %s ausgegeben!%n", kopiXml);
      schreib(jdomDoc, kopiXml);
      printf("Die Datei %s hat folgenden Inhalt:%n", kopiXml);
      printf("--------------------------------------------------------%n");
      gibTextDateiAus(kopiXml);
      printf("--------------------------------------------------------%n");
      printf("Das (aus der Datei %s erzeugte) JDOM-Objekt%n", origXml);
      printf("wird mit isGueltigVD validiert. ");
      printf("Ergebnis: %b%n", istGueltigVD(jdomDoc));
      printf("Das (aus der Datei %s erzeugte) JDOM-Objekt%n", origXml);
      printf("wird mit isGueltigVS validiert. ");
      printf("Ergebnis: %b%n", istGueltigVS(jdomDoc, pfadXsd));
      printf("********************************************************%n");

      // Problem: Wenn man das jdomDoc mit der Methode istGueltigVS3
      // validiert, wird es in ein domDoc1 umgewandelt und dieses wird mit
      // XmlJStd.istGueltigVS validiert. Ergebnis: false.
      // Wenn man das domDoc in eine Datei domF schreibt, diese Datei
      // als DOM-Objekt einliest und ebenfalls mit XmlJStd.istGueltigVS
      // validiert, ist das Ergebnis true. Wieso??

      printf("Das JDOM-Objekt wird  gegen %s validiert:%n", pfadXsd);
      boolean erg1 = istGueltigVS3(jdomDoc, pfadXsd);
      printf("Das JDOM-Objekt wurde gegen %s validiert:%n", pfadXsd);
      printf("Ergebnis der Validierung: %b%n%n", erg1);
      printf("XmlJStd.istGueltigVS(XmlJStd.lies(domF), %s):%n", pfadXsd);
      boolean erg2 = XmlJStd.istGueltigVS(XmlJStd.lies(domF), pfadXsd);
      printf("Ergebnis der Validierung: %b%n", erg2);
      printf("********************************************************%n");

      printf("XmlJdom: Das war's erstmal!%n");
   }  // main
   // ---------------------------------------------------------------------
   // Eine Methode mit einem kurzen Namen:
   static void printf(String f, Object... v) {System.out.printf(f, v);}
   // ---------------------------------------------------------------------
} // class XmlJdom
/* ------------------------------------------------------------------------
Die Ausgabe des Programms XmlJdom.java (2007-10-03):

XmlJdom: Jetzt geht es los!
********************************************************
Die Datei daten01B.xml hat folgenden Inhalt:
--------------------------------------------------------
<?xml version="1.0" encoding="UTF-8"?>
<!-- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
   Datei daten01B.xml:
   Wohlgeformt und gueltig (valid) nach nameA.dtd und nameA.xsd.
   Enthaelt einen Hinweis auf name.dtd
- - - - - - - - - - - - - - - - - - - - - - - - - - - - - -  -->

<!DOCTYPE name SYSTEM "nameA.dtd">

<name>

  <vor_name> Alan    </vor_name>

  <nach_name>Turing  </nach_name>
</name>
--------------------------------------------------------
Die Datei daten01B.xml wird in ein JDOM-Objekt eingelesen!
Das JDOM-Objekt wird in die Datei dJdom01B.xml ausgegeben!
Die Datei dJdom01B.xml hat folgenden Inhalt:
--------------------------------------------------------
<?xml version="1.0" encoding="UTF-8"?>
<!-- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
   Datei daten01B.xml:
   Wohlgeformt und gueltig (valid) nach nameA.dtd und nameA.xsd.
   Enthaelt einen Hinweis auf name.dtd
- - - - - - - - - - - - - - - - - - - - - - - - - - - - - -  -->
<!DOCTYPE name SYSTEM "nameA.dtd">

<name>
  <vor_name>Alan</vor_name>
  <nach_name>Turing</nach_name>
</name>

--------------------------------------------------------
Das (aus der Datei daten01B.xml erzeugte) JDOM-Objekt
wird mit isGueltigVD validiert. Ergebnis: true
Das (aus der Datei daten01B.xml erzeugte) JDOM-Objekt
wird mit isGueltigVS validiert. Ergebnis: true
********************************************************
Das JDOM-Objekt wird  gegen nameA.xsd validiert:
+++++++++++++++++++++++++++++++++++++++++++++++++++
**0** XmlJdom.istGueltigVS3 ruft XmlJStdVS.istGueltigVS auf:
+++++++++++++++++++++++++++++++++++++++++++++++++++
**1** Das originale JDOM-Dokument jdomDoc:

<?xml version="1.0" encoding="UTF-8"?>
<!-- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
   Datei daten01B.xml:
   Wohlgeformt und gueltig (valid) nach nameA.dtd und nameA.xsd.
   Enthaelt einen Hinweis auf name.dtd
- - - - - - - - - - - - - - - - - - - - - - - - - - - - - -  -->
<!DOCTYPE name SYSTEM "nameA.dtd">

<name>
  <vor_name>Alan</vor_name>
  <nach_name>Turing</nach_name>
</name>

+++++++++++++++++++++++++++++++++++++++++++++++++++
**2** Das entsprechende DOM-Objekt domDoc:

<?xml version="1.0" encoding="UTF-8"?>
<!DOCTYPE name SYSTEM "nameA.dtd">
<name>
    <vor_name> Alan    </vor_name>
    <nach_name>Turing  </nach_name>
</name>
<!-- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
   Datei daten01B.xml:
   Wohlgeformt und gueltig (valid) nach nameA.dtd und nameA.xsd.
   Enthaelt einen Hinweis auf name.dtd
- - - - - - - - - - - - - - - - - - - - - - - - - - - - - -  -->
+++++++++++++++++++++++++++++++++++++++++++++++++++
**3** Validiert wird gegen die Schema-Datei nameA.xsd:

<?xml version="1.0" encoding="UTF-8"?>
<!-- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
   Datei nameA.xsd
   XML-Schema-Datei fuer die Dateien daten01x.xml
- - - - - - - - - - - - - - - - - - - - - - - - - - - - - -  -->
<xsd:schema xmlns:xsd="http://www.w3.org/2001/XMLSchema"
  elementFormDefault="qualified">
  <xsd:element name="name">

    <xsd:complexType>
       <xsd:sequence>
          <xsd:element name="vor_name"  type="xsd:string"/>
          <xsd:element name="nach_name" type="xsd:string"/>
       </xsd:sequence>
    </xsd:complexType>

  </xsd:element>
</xsd:schema>
+++++++++++++++++++++++++++++++++++++++++++++++++++
In XmlJStd.istGueltigVS: org.xml.sax.SAXParseException
Message: cvc-elt.1: Cannot find the declaration of element 'name'.
Das JDOM-Objekt wurde gegen nameA.xsd validiert:
Ergebnis der Validierung: false

XmlJStd.istGueltigVS(XmlJStd.lies(domF), nameA.xsd):
Ergebnis der Validierung: true
********************************************************
XmlJdom: Das war's erstmal!
------------------------------------------------------------------------ */
