// Datei XmlXerces.java
/* ------------------------------------------------------------------------
Dieser Modul enthaelt Methoden zum Bearbeiten von XML-Dateien und
benutzt dazu Klassen des Xerces-Projekts (siehe die Netzadresse
http://xerces.apache.org/xerces2-j/).

Die wichtige Methode:
schreib          Schreibt ein DOM-Document-Objekt im XML-Format
                 in eine Datei

Weniger wichtige Methode:
gibTextDateiAus  Gibt eine beliebige Textdatei zur
                 Standardausgabe aus
---------------------------------------------------------------------------
Ausserdem enthaelt dieser Modul eine main-Methode, die einige der
oben erwaehnten Methoden ein bisschen testet.
------------------------------------------------------------------------ */
import java .io                  .FileReader;
import java .io                  .FileWriter;
import org  .w3c   .dom          .Document;
import org  .apache.xml.serialize.OutputFormat;
import org  .apache.xml.serialize.XMLSerializer;

class XmlXerces {
   // ---------------------------------------------------------------------
   public static void schreib(Document domDoc, String pfad) {
      try {
         OutputFormat format = new OutputFormat(domDoc);
         format.setLineWidth(75);
         format.setIndenting(true);
         format.setIndent(3);

         FileWriter fw = new FileWriter(pfad);
         XMLSerializer ser = new XMLSerializer(fw, format);
         ser.serialize(domDoc);
      } catch (Exception ex) {
         meldeAusnahme(ex, "schreib");
      }
   } // schreib
   // ---------------------------------------------------------------------
   static public void gibTextDateiAus(String pfad) {
      // Gibt die Textdatei mit dem Pfad pfad zur Standardausgabe aus.
      try {
         FileReader fr = new FileReader(pfad);
         int zeichen;
         while ((zeichen = fr.read()) != -1) printf("%c", zeichen);
         fr.close();
      } catch (Exception ex) {
         meldeAusnahme(ex, "gibTextDateiAus");
      }
   } // gibTextDateiAus
   // ---------------------------------------------------------------------
   static public void meldeAusnahme(Exception ex, String inMethode) {
      String ausnahme = ex.getClass().getName();
      printf("In XmlXerces.%s: %s%n", inMethode, ausnahme);
      printf("Message: %s%n", ex.getMessage());
   } // meldeAusnahme
   // ---------------------------------------------------------------------
   static public void main(String[] sonja) {
      printf("XmlXerces: Jetzt geht es los!%n");
      printf("********************************************************%n");

      String orig = "daten01A.xml";
      String kopi = "dXerc01A.xml";

      printf("Die Datei %s hat folgenden Inhalt:%n", orig);
      printf("--------------------------------------------------------%n");
      gibTextDateiAus(orig);
      printf("--------------------------------------------------------%n");
      printf("Die Datei %s wird in ein DOM-Objekt eingelesen!%n", orig);
      Document domDoc = XmlJStd.lies(orig);
      printf("Das DOM-Objekt wird in die Datei %s ausgegeben!%n", kopi);
      schreib(domDoc, kopi);
      printf("Die Datei %s hat folgenden Inhalt:%n", kopi);
      printf("--------------------------------------------------------%n");
      gibTextDateiAus(kopi);
      printf("********************************************************%n");
      printf("XmlXerces: Das war's erstmal!%n");
   } // main
   // ---------------------------------------------------------------------
   // Eine Methode mit einem kurzen Namen:
   static void printf(String f, Object... v) {System.out.printf(f, v);}
   // ---------------------------------------------------------------------
} // class XmlXerces
/* ------------------------------------------------------------------------
Ausgabe dieses Programms:

XmlXerces: Jetzt geht es los!
********************************************************
Die Datei daten01A.xml hat folgenden Inhalt:
--------------------------------------------------------
<?xml version="1.0" encoding="UTF-8"?>
<!-- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
   Datei daten01A.xml:
   Wohlgeformt und gueltig (valid) nach name.dtd und name.xsd.
   Enthaelt keinen Hinweis auf name.dtd oder name.xsd
- - - - - - - - - - - - - - - - - - - - - - - - - - - - - -  -->


<name>

  <vor_name> Alan  </vor_name>
  <nach_name>Turing</nach_name>
</name>

--------------------------------------------------------
Die Datei daten01A.xml wird in ein DOM-Objekt eingelesen!
Das DOM-Objekt wird in die Datei dXerc01A.xml ausgegeben!
Die Datei dXerc01A.xml hat folgenden Inhalt:
--------------------------------------------------------
<?xml version="1.0" encoding="UTF-8"?>
<!-- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
   Datei daten01A.xml:
   Wohlgeformt und gueltig (valid) nach name.dtd und name.xsd.
   Enthaelt keinen Hinweis auf name.dtd oder name.xsd
- - - - - - - - - - - - - - - - - - - - - - - - - - - - - -  -->
<name>
   <vor_name> Alan  </vor_name>
   <nach_name>Turing</nach_name>
</name>
********************************************************
XmlXerces: Das war's erstmal!
------------------------------------------------------------------------ */
