// Datei Reflektion01.java
/* ------------------------------------------------------------------------
Dieses Programm prueft, ob es bestimmte Permissionen (engl. permissions)
hat oder nicht.
------------------------------------------------------------------------ */
import java.security    .Permission;
import java.security    .AllPermission;
import java.util        .PropertyPermission;
import java.lang.reflect.ReflectPermission;
import java.security    .AccessControlException;
import java.lang        .RuntimePermission;

class Reflektion01 {
   // ---------------------------------------------------------------------
   static private void pruefe(Permission p) {
      // Prueft, ob das aktuelle Programm (zu dem dieses Unterprogramm
      // gehoert) die Permission p hat oder nicht. Gibt eine entsprechende
      // Meldung zur Standardausgabe aus.

      try {
         p.checkGuard(null);            // Der Parameter wird nicht benutzt
         printf("%-64s HAS been granted!%n", p);
      } catch (SecurityException sex) {
         printf("%-64s has NOT been granted!%n", p);
      }
   } // pruefe
   // ---------------------------------------------------------------------
   static public void main(String[] _) {
      printf("Reflektion01: Jetzt geht es los!%n");
      printf("================================%n");

      // Eine Reihung von Permissionsobjekten:
      Permission[] pr = {
         new PropertyPermission("java.version", "read" ),
         new PropertyPermission("java.version", "write"),
         new PropertyPermission("java.home",    "read" ),
         new PropertyPermission("java.home",    "write"),
         new PropertyPermission("user.home",    "read" ),
         new PropertyPermission("user.home",    "write"),
         new ReflectPermission ("suppressAccessChecks" ),
         new RuntimePermission ("accessDeclaredMembers"),
         new AllPermission     (),
      };

      for (Permission p : pr) pruefe(p);

      printf("================================%n");
      printf("Reflektion01: Das war's erstmal!%n");
   } // main
   // ---------------------------------------------------------------------
   // Eine Methode mit einem kurzen Namen:
   static void printf(String f, Object... v) {System.out.printf(f, v);}
   // ---------------------------------------------------------------------
} // class Reflektion01
/* ------------------------------------------------------------------------
Wenn man dieses Programm mit dem Kommando

> java Reflektion01

startet, wird es OHNE ein SecurityManager-Objekt ausgefuehrt und hat damit
alle moeglichen Permissionen. In diesem Fall gibt das Programm die folgenden
Zeilen aus:

Reflektion01: Jetzt geht es los!
================================
(java.util.PropertyPermission java.version read)                 HAS been granted!
(java.util.PropertyPermission java.version write)                HAS been granted!
(java.util.PropertyPermission java.home read)                    HAS been granted!
(java.util.PropertyPermission java.home write)                   HAS been granted!
(java.util.PropertyPermission user.home read)                    HAS been granted!
(java.util.PropertyPermission user.home write)                   HAS been granted!
(java.lang.reflect.ReflectPermission suppressAccessChecks)       HAS been granted!
(java.lang.RuntimePermission accessDeclaredMembers)              HAS been granted!
(java.security.AllPermission <all permissions> <all actions>)    HAS been granted!
================================
Reflektion01: Das war's erstmal!

Wenn man dieses Programm dagegen mit dem Kommando

 > java -Djava.security.manager -Djava.security.policy==meineJavaPermissionen Reflektion01

startet, wird es MIT einem SecurityManager-Objekt ausfuehrt. In diesem
Fall bestimmt der Inhalt der Datei meineJavaPermissionen, welche
Permissionen das Programm hat. Angenommen, diese Datei hat folgenden Inhalt:

// -------------------------------------------------------------------------
// Datei meineJavaPermissionen

grant {
   permission java.util.PropertyPermission "java.version",       "read";
   permission java.util.PropertyPermission "java.home",          "read";
   permission java.util.PropertyPermission "user.home",          "read";
   permission java.util.PropertyPermission "line.separator",     "read";
   permission java.lang.reflect.ReflectPermission  "suppressAccessChecks";
   permission java.lang.RuntimePermission  "accessDeclaredMembers";
// permission java.security.AllPermission;
};
// -------------------------------------------------------------------------

Dann gibt das Programm Reflektion01 folgende Zeilen aus:

Reflektion01: Jetzt geht es los!
================================
(java.util.PropertyPermission java.version read)                 HAS been granted!
(java.util.PropertyPermission java.version write)                has NOT been granted!
(java.util.PropertyPermission java.home read)                    HAS been granted!
(java.util.PropertyPermission java.home write)                   has NOT been granted!
(java.util.PropertyPermission user.home read)                    HAS been granted!
(java.util.PropertyPermission user.home write)                   has NOT been granted!
(java.lang.reflect.ReflectPermission suppressAccessChecks)       HAS been granted!
(java.lang.RuntimePermission accessDeclaredMembers)              HAS been granted!
(java.security.AllPermission <all permissions> <all actions>)    has NOT been granted!
================================
Reflektion01: Das war's erstmal!
------------------------------------------------------------------------ */
