// Datei Reflektion03.java
/* ------------------------------------------------------------------------
Dieses Programm prueft, ob es bestimmte System-Eigenschaften (engl. system
properties) lesen darf oder nicht. Wenn es sie lesen darf, gibt es ihren
Wert aus (und sonst den Text "No read permission").
------------------------------------------------------------------------ */
import java.util.PropertyPermission;

class Reflektion03 {
   // ---------------------------------------------------------------------
   // Eine Reihung von Namen von System-Eigenschaften (system properties):
   static private String[] nameR = {
      "awt.toolkit",
      "file.encoding",
      "file.encoding.pkg",
      "file.separator",
      "java.awt.graphicsenv",
      "java.awt.printerjob",
      "java.class.version",
      "java.class.path",
      "java.endorsed.dirs",
      "java.ext.dirs",
      "java.home",
      "java.io.tmpdir",
      "java.library.path",
      "java.runtime.version",
      "java.runtime.name",
      "java.specification.name",
      "java.specification.vendor",
      "java.specification.version",
      "java.vendor",
      "java.vendor.url.bug",
      "java.vendor.url",
      "java.version",
      "java.vm.info",
      "java.vm.name",
      "java.vm.specification.version",
      "java.vm.specification.name",
      "java.vm.specification.vendor",
      "java.vm.vendor",
      "java.vm.version",
      "line.separator",
      "os.arch",
      "os.name",
      "os.version",
      "path.separator",
      "sun.arch.data.model",
      "sun.boot.class.path",
      "sun.boot.library.path",
      "sun.cpu.endian",
      "sun.cpu.isalist",
      "sun.desktop",
      "sun.io.unicode.encoding",
      "sun.java.launcher",
      "sun.jnu.encoding",
      "sun.management.compiler",
      "sun.os.patch.level",
      "user.country",
      "user.dir",
      "user.home",
      "user.language",
      "user.name",
      "user.timezone",
      "user.variant",
   }; // String[] nameR
   // ---------------------------------------------------------------------
   static public void printOneSystemProperty(String name) {
      // Gibt den name aus. Und falls dieses Programm die Permission
      // hat, eine System-Eigenschaft (system property) mit diesem Namen
      // zu lesen, wird auch der Wert dieser Eigenschaft ausgegeben, sonst
      // der Text "No read permission!".
      try {
         printf("|%-30s|", name);
         PropertyPermission pp = new PropertyPermission(name, "read");

         // Der folgende Befehl wirft eine Ausnahme, wenn dieses Programm
         // die PropertyPermission NICHT hat:
         pp.checkGuard(null);

         // Wenn keine Ausnahme geworfen wurde:
         String pv = System.getProperty(name);
         printf("\"%s\"%n", pv);
      } catch (SecurityException sex) {
         printf("No read permission!%n");
      }
   } // printOneSystemProperty
   // ---------------------------------------------------------------------
   static public void main(String[] sonja) {
      printf("Reflektion03: Jetzt geht es los!%n");
      printf("================================%n");

      String mz = "------------------------------------------------------";

      printf("|%-30s|%s%n", "Name der Poperty:", "Wert der Property:");
      printf("+%.30s+%s%n", mz, mz);
      for (String name : nameR) printOneSystemProperty(name);
      printf("+%.30s+%s%n", mz, mz);
      printf("================================%n");
      printf("Reflektion03: Das war's erstmal!%n");
   } // main
   // ---------------------------------------------------------------------
   // Eine Methode mit einem kurzen Namen:
   static void printf(String f, Object... v) {System.out.printf(f, v);}
   // ---------------------------------------------------------------------
} // class Reflektion03
/* ------------------------------------------------------------------------
Wenn man dieses Programm mit dem folgenden Kommando startet:

> java -Djava.security.manager -Djava.security.policy==meineJavaPermissionen Reflektion03

hat es nur die in der Datei meineJavaPermissionen gewaehrten Permissionen.
Hier wird angenommen, dass diese Datei folgenden Inhalt hat:
---------------------------------------------------------------------------
// Datei meineJavaPermissionen

grant {

   permission java.util.PropertyPermission "java.version",   "read";
   permission java.util.PropertyPermission "java.home",      "read";
   permission java.util.PropertyPermission "user.home",      "read";
   permission java.util.PropertyPermission "os.version",     "read";

   permission java.util.PropertyPermission "line.separator", "read";
};
---------------------------------------------------------------------------
Die Ausgabe des Programms Reflektion03 kann dann etwa so aussehen:

Reflektion03: Jetzt geht es los!
================================
|Name der Poperty:             |Wert der Property:
+------------------------------+------------------------------------------------------
|awt.toolkit                   |No read permission!
|file.encoding                 |No read permission!
|file.encoding.pkg             |No read permission!
|file.separator                |No read permission!
|java.awt.graphicsenv          |No read permission!
|java.awt.printerjob           |No read permission!
|java.class.version            |No read permission!
|java.class.path               |No read permission!
|java.endorsed.dirs            |No read permission!
|java.ext.dirs                 |No read permission!
|java.home                     |"C:\Programme\Java\jre1.6.0_07"
|java.io.tmpdir                |No read permission!
|java.library.path             |No read permission!
|java.runtime.version          |No read permission!
|java.runtime.name             |No read permission!
|java.specification.name       |No read permission!
|java.specification.vendor     |No read permission!
|java.specification.version    |No read permission!
|java.vendor                   |No read permission!
|java.vendor.url.bug           |No read permission!
|java.vendor.url               |No read permission!
|java.version                  |"1.6.0_07"
|java.vm.info                  |No read permission!
|java.vm.name                  |No read permission!
|java.vm.specification.version |No read permission!
|java.vm.specification.name    |No read permission!
|java.vm.specification.vendor  |No read permission!
|java.vm.vendor                |No read permission!
|java.vm.version               |No read permission!
|line.separator                |"
"
|os.arch                       |No read permission!
|os.name                       |No read permission!
|os.version                    |"5.1"
|path.separator                |No read permission!
|sun.arch.data.model           |No read permission!
|sun.boot.class.path           |No read permission!
|sun.boot.library.path         |No read permission!
|sun.cpu.endian                |No read permission!
|sun.cpu.isalist               |No read permission!
|sun.desktop                   |No read permission!
|sun.io.unicode.encoding       |No read permission!
|sun.java.launcher             |No read permission!
|sun.jnu.encoding              |No read permission!
|sun.management.compiler       |No read permission!
|sun.os.patch.level            |No read permission!
|user.country                  |No read permission!
|user.dir                      |No read permission!
|user.home                     |"C:\Dokumente und Einstellungen\Ulrich"
|user.language                 |No read permission!
|user.name                     |No read permission!
|user.timezone                 |No read permission!
|user.variant                  |No read permission!
+------------------------------+------------------------------------------------------
================================
Reflektion03: Das war's erstmal!
---------------------------------------------------------------------------
Hinweis: Die Ausfuehrung eines Befehls wie  printf("%n");
erfordert die PropertyPermission namens "line.separator", weil %n durch
den Zeilentrenner (line separator) der aktuell verwendeten Plattform
ersetzt wird ("\n" unter Unix, "\r" unter Mac OS und "\n\r" unter Windows).
Dazu ist ein lesender Zugriff auf die Eigenschaft (property) namens
line.separator erforderlich.
------------------------------------------------------------------------ */
